{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.CSparseCholesky
       (
         CSparseCholesky,
         CSparseCholeskyClass(..),
         csparseCholesky,
         csparseCholesky',
         csparseCholesky'',
         csparseCholesky_checkNode,
         csparseCholesky_creator,
         csparseCholesky_getFactorization,
         csparseCholesky_getFactorization',
         csparseCholesky_getFactorizationSparsity,
         csparseCholesky_getFactorizationSparsity',
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show CSparseCholesky where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__checkNode" c_CasADi__CSparseCholesky__checkNode
  :: Ptr CSparseCholesky' -> IO CInt
casADi__CSparseCholesky__checkNode
  :: CSparseCholesky -> IO Bool
casADi__CSparseCholesky__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CSparseCholesky__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
csparseCholesky_checkNode :: CSparseCholeskyClass a => a -> IO Bool
csparseCholesky_checkNode x = casADi__CSparseCholesky__checkNode (castCSparseCholesky x)


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__getFactorizationSparsity" c_CasADi__CSparseCholesky__getFactorizationSparsity
  :: Ptr CSparseCholesky' -> CInt -> IO (Ptr Sparsity')
casADi__CSparseCholesky__getFactorizationSparsity
  :: CSparseCholesky -> Bool -> IO Sparsity
casADi__CSparseCholesky__getFactorizationSparsity x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__CSparseCholesky__getFactorizationSparsity x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Obtain a symbolic Cholesky factorization.
-}
csparseCholesky_getFactorizationSparsity :: CSparseCholeskyClass a => a -> Bool -> IO Sparsity
csparseCholesky_getFactorizationSparsity x = casADi__CSparseCholesky__getFactorizationSparsity (castCSparseCholesky x)


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__getFactorizationSparsity_TIC" c_CasADi__CSparseCholesky__getFactorizationSparsity_TIC
  :: Ptr CSparseCholesky' -> IO (Ptr Sparsity')
casADi__CSparseCholesky__getFactorizationSparsity'
  :: CSparseCholesky -> IO Sparsity
casADi__CSparseCholesky__getFactorizationSparsity' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CSparseCholesky__getFactorizationSparsity_TIC x0' >>= wrapReturn

-- classy wrapper
csparseCholesky_getFactorizationSparsity' :: CSparseCholeskyClass a => a -> IO Sparsity
csparseCholesky_getFactorizationSparsity' x = casADi__CSparseCholesky__getFactorizationSparsity' (castCSparseCholesky x)


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__getFactorization" c_CasADi__CSparseCholesky__getFactorization
  :: Ptr CSparseCholesky' -> CInt -> IO (Ptr DMatrix')
casADi__CSparseCholesky__getFactorization
  :: CSparseCholesky -> Bool -> IO DMatrix
casADi__CSparseCholesky__getFactorization x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__CSparseCholesky__getFactorization x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Obtain a numeric Cholesky factorization.
-}
csparseCholesky_getFactorization :: CSparseCholeskyClass a => a -> Bool -> IO DMatrix
csparseCholesky_getFactorization x = casADi__CSparseCholesky__getFactorization (castCSparseCholesky x)


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__getFactorization_TIC" c_CasADi__CSparseCholesky__getFactorization_TIC
  :: Ptr CSparseCholesky' -> IO (Ptr DMatrix')
casADi__CSparseCholesky__getFactorization'
  :: CSparseCholesky -> IO DMatrix
casADi__CSparseCholesky__getFactorization' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CSparseCholesky__getFactorization_TIC x0' >>= wrapReturn

-- classy wrapper
csparseCholesky_getFactorization' :: CSparseCholeskyClass a => a -> IO DMatrix
csparseCholesky_getFactorization' x = casADi__CSparseCholesky__getFactorization' (castCSparseCholesky x)


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__creator" c_CasADi__CSparseCholesky__creator
  :: Ptr Sparsity' -> CInt -> IO (Ptr LinearSolver')
casADi__CSparseCholesky__creator
  :: Sparsity -> Int -> IO LinearSolver
casADi__CSparseCholesky__creator x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__CSparseCholesky__creator x0' x1' >>= wrapReturn

-- classy wrapper
csparseCholesky_creator :: Sparsity -> Int -> IO LinearSolver
csparseCholesky_creator = casADi__CSparseCholesky__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__CSparseCholesky" c_CasADi__CSparseCholesky__CSparseCholesky
  :: IO (Ptr CSparseCholesky')
casADi__CSparseCholesky__CSparseCholesky
  :: IO CSparseCholesky
casADi__CSparseCholesky__CSparseCholesky  =
  c_CasADi__CSparseCholesky__CSparseCholesky  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::CSparseCholesky::CSparseCholesky()
>------------------------------------------------------------------------
>
>Default (empty) constructor.
>
>>  CasADi::CSparseCholesky::CSparseCholesky(const Sparsity &sp, int nrhs=1)
>------------------------------------------------------------------------
>
>Create a linear solver given a sparsity pattern.
-}
csparseCholesky :: IO CSparseCholesky
csparseCholesky = casADi__CSparseCholesky__CSparseCholesky


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__CSparseCholesky_TIC" c_CasADi__CSparseCholesky__CSparseCholesky_TIC
  :: Ptr Sparsity' -> CInt -> IO (Ptr CSparseCholesky')
casADi__CSparseCholesky__CSparseCholesky'
  :: Sparsity -> Int -> IO CSparseCholesky
casADi__CSparseCholesky__CSparseCholesky' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__CSparseCholesky__CSparseCholesky_TIC x0' x1' >>= wrapReturn

-- classy wrapper
csparseCholesky' :: Sparsity -> Int -> IO CSparseCholesky
csparseCholesky' = casADi__CSparseCholesky__CSparseCholesky'


-- direct wrapper
foreign import ccall unsafe "CasADi__CSparseCholesky__CSparseCholesky_TIC_TIC" c_CasADi__CSparseCholesky__CSparseCholesky_TIC_TIC
  :: Ptr Sparsity' -> IO (Ptr CSparseCholesky')
casADi__CSparseCholesky__CSparseCholesky''
  :: Sparsity -> IO CSparseCholesky
casADi__CSparseCholesky__CSparseCholesky'' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CSparseCholesky__CSparseCholesky_TIC_TIC x0' >>= wrapReturn

-- classy wrapper
csparseCholesky'' :: Sparsity -> IO CSparseCholesky
csparseCholesky'' = casADi__CSparseCholesky__CSparseCholesky''

