{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.CasadiOptions
       (
         CasadiOptions,
         CasadiOptionsClass(..),
         casadiOptions_getAllowedInternalAPI,
         casadiOptions_getCatchErrorsPython,
         casadiOptions_getProfilingBinary,
         casadiOptions_getSimplificationOnTheFly,
         casadiOptions_setAllowedInternalAPI,
         casadiOptions_setCatchErrorsPython,
         casadiOptions_setProfilingBinary,
         casadiOptions_setPurgeSeeds,
         casadiOptions_setPurgeSeeds',
         casadiOptions_setSimplificationOnTheFly,
         casadiOptions_startProfiling,
         casadiOptions_stopProfiling,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__setCatchErrorsPython" c_CasADi__CasadiOptions__setCatchErrorsPython
  :: CInt -> IO ()
casADi__CasadiOptions__setCatchErrorsPython
  :: Bool -> IO ()
casADi__CasadiOptions__setCatchErrorsPython x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CasadiOptions__setCatchErrorsPython x0' >>= wrapReturn

-- classy wrapper
casadiOptions_setCatchErrorsPython :: Bool -> IO ()
casadiOptions_setCatchErrorsPython = casADi__CasadiOptions__setCatchErrorsPython


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__getCatchErrorsPython" c_CasADi__CasadiOptions__getCatchErrorsPython
  :: IO CInt
casADi__CasadiOptions__getCatchErrorsPython
  :: IO Bool
casADi__CasadiOptions__getCatchErrorsPython  =
  c_CasADi__CasadiOptions__getCatchErrorsPython  >>= wrapReturn

-- classy wrapper
casadiOptions_getCatchErrorsPython :: IO Bool
casadiOptions_getCatchErrorsPython = casADi__CasadiOptions__getCatchErrorsPython


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__setSimplificationOnTheFly" c_CasADi__CasadiOptions__setSimplificationOnTheFly
  :: CInt -> IO ()
casADi__CasadiOptions__setSimplificationOnTheFly
  :: Bool -> IO ()
casADi__CasadiOptions__setSimplificationOnTheFly x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CasadiOptions__setSimplificationOnTheFly x0' >>= wrapReturn

-- classy wrapper
casadiOptions_setSimplificationOnTheFly :: Bool -> IO ()
casadiOptions_setSimplificationOnTheFly = casADi__CasadiOptions__setSimplificationOnTheFly


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__getSimplificationOnTheFly" c_CasADi__CasadiOptions__getSimplificationOnTheFly
  :: IO CInt
casADi__CasadiOptions__getSimplificationOnTheFly
  :: IO Bool
casADi__CasadiOptions__getSimplificationOnTheFly  =
  c_CasADi__CasadiOptions__getSimplificationOnTheFly  >>= wrapReturn

-- classy wrapper
casadiOptions_getSimplificationOnTheFly :: IO Bool
casadiOptions_getSimplificationOnTheFly = casADi__CasadiOptions__getSimplificationOnTheFly


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__startProfiling" c_CasADi__CasadiOptions__startProfiling
  :: Ptr StdString' -> IO ()
casADi__CasadiOptions__startProfiling
  :: String -> IO ()
casADi__CasadiOptions__startProfiling x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CasadiOptions__startProfiling x0' >>= wrapReturn

-- classy wrapper
casadiOptions_startProfiling :: String -> IO ()
casadiOptions_startProfiling = casADi__CasadiOptions__startProfiling


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__stopProfiling" c_CasADi__CasadiOptions__stopProfiling
  :: IO ()
casADi__CasadiOptions__stopProfiling
  :: IO ()
casADi__CasadiOptions__stopProfiling  =
  c_CasADi__CasadiOptions__stopProfiling  >>= wrapReturn

-- classy wrapper
casadiOptions_stopProfiling :: IO ()
casadiOptions_stopProfiling = casADi__CasadiOptions__stopProfiling


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__setProfilingBinary" c_CasADi__CasadiOptions__setProfilingBinary
  :: CInt -> IO ()
casADi__CasadiOptions__setProfilingBinary
  :: Bool -> IO ()
casADi__CasadiOptions__setProfilingBinary x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CasadiOptions__setProfilingBinary x0' >>= wrapReturn

-- classy wrapper
casadiOptions_setProfilingBinary :: Bool -> IO ()
casadiOptions_setProfilingBinary = casADi__CasadiOptions__setProfilingBinary


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__getProfilingBinary" c_CasADi__CasadiOptions__getProfilingBinary
  :: IO CInt
casADi__CasadiOptions__getProfilingBinary
  :: IO Bool
casADi__CasadiOptions__getProfilingBinary  =
  c_CasADi__CasadiOptions__getProfilingBinary  >>= wrapReturn

-- classy wrapper
casadiOptions_getProfilingBinary :: IO Bool
casadiOptions_getProfilingBinary = casADi__CasadiOptions__getProfilingBinary


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__setPurgeSeeds" c_CasADi__CasadiOptions__setPurgeSeeds
  :: CInt -> IO ()
casADi__CasadiOptions__setPurgeSeeds
  :: Bool -> IO ()
casADi__CasadiOptions__setPurgeSeeds x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CasadiOptions__setPurgeSeeds x0' >>= wrapReturn

-- classy wrapper
casadiOptions_setPurgeSeeds :: Bool -> IO ()
casadiOptions_setPurgeSeeds = casADi__CasadiOptions__setPurgeSeeds


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__setPurgeSeeds_TIC" c_CasADi__CasadiOptions__setPurgeSeeds_TIC
  :: IO CInt
casADi__CasadiOptions__setPurgeSeeds'
  :: IO Bool
casADi__CasadiOptions__setPurgeSeeds'  =
  c_CasADi__CasadiOptions__setPurgeSeeds_TIC  >>= wrapReturn

-- classy wrapper
casadiOptions_setPurgeSeeds' :: IO Bool
casadiOptions_setPurgeSeeds' = casADi__CasadiOptions__setPurgeSeeds'


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__setAllowedInternalAPI" c_CasADi__CasadiOptions__setAllowedInternalAPI
  :: CInt -> IO ()
casADi__CasadiOptions__setAllowedInternalAPI
  :: Bool -> IO ()
casADi__CasadiOptions__setAllowedInternalAPI x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__CasadiOptions__setAllowedInternalAPI x0' >>= wrapReturn

-- classy wrapper
casadiOptions_setAllowedInternalAPI :: Bool -> IO ()
casadiOptions_setAllowedInternalAPI = casADi__CasadiOptions__setAllowedInternalAPI


-- direct wrapper
foreign import ccall unsafe "CasADi__CasadiOptions__getAllowedInternalAPI" c_CasADi__CasadiOptions__getAllowedInternalAPI
  :: IO CInt
casADi__CasadiOptions__getAllowedInternalAPI
  :: IO Bool
casADi__CasadiOptions__getAllowedInternalAPI  =
  c_CasADi__CasadiOptions__getAllowedInternalAPI  >>= wrapReturn

-- classy wrapper
casadiOptions_getAllowedInternalAPI :: IO Bool
casadiOptions_getAllowedInternalAPI = casADi__CasadiOptions__getAllowedInternalAPI

