{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.DpleSolver
       (
         DpleSolver,
         DpleSolverClass(..),
         dpleSolver,
         dpleSolver_checkNode,
         dpleSolver_clone,
         dpleSolver_printStats',
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show DpleSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__DpleSolver__clone" c_CasADi__DpleSolver__clone
  :: Ptr DpleSolver' -> IO (Ptr DpleSolver')
casADi__DpleSolver__clone
  :: DpleSolver -> IO DpleSolver
casADi__DpleSolver__clone x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__DpleSolver__clone x0' >>= wrapReturn

-- classy wrapper
{-|
>Clone.
-}
dpleSolver_clone :: DpleSolverClass a => a -> IO DpleSolver
dpleSolver_clone x = casADi__DpleSolver__clone (castDpleSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__DpleSolver__printStats_TIC" c_CasADi__DpleSolver__printStats_TIC
  :: Ptr DpleSolver' -> IO ()
casADi__DpleSolver__printStats'
  :: DpleSolver -> IO ()
casADi__DpleSolver__printStats' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__DpleSolver__printStats_TIC x0' >>= wrapReturn

-- classy wrapper
dpleSolver_printStats' :: DpleSolverClass a => a -> IO ()
dpleSolver_printStats' x = casADi__DpleSolver__printStats' (castDpleSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__DpleSolver__checkNode" c_CasADi__DpleSolver__checkNode
  :: Ptr DpleSolver' -> IO CInt
casADi__DpleSolver__checkNode
  :: DpleSolver -> IO Bool
casADi__DpleSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__DpleSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
dpleSolver_checkNode :: DpleSolverClass a => a -> IO Bool
dpleSolver_checkNode x = casADi__DpleSolver__checkNode (castDpleSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__DpleSolver__DpleSolver" c_CasADi__DpleSolver__DpleSolver
  :: IO (Ptr DpleSolver')
casADi__DpleSolver__DpleSolver
  :: IO DpleSolver
casADi__DpleSolver__DpleSolver  =
  c_CasADi__DpleSolver__DpleSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
dpleSolver :: IO DpleSolver
dpleSolver = casADi__DpleSolver__DpleSolver

