{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.ExpDMatrix
       (
         ExpDMatrix,
         ExpDMatrixClass(..),
         expDMatrix___ge__,
         expDMatrix___gt__,
         expDMatrix___mldivide__,
         expDMatrix___radd__,
         expDMatrix___rdiv__,
         expDMatrix___req__,
         expDMatrix___rge__,
         expDMatrix___rgt__,
         expDMatrix___rle__,
         expDMatrix___rlt__,
         expDMatrix___rmul__,
         expDMatrix___rne__,
         expDMatrix___rsub__,
         expDMatrix___rtruediv__,
         expDMatrix___truediv__,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______mldivide__" c_CasADi__GenericExpression_CasADi__Matrix_double______mldivide__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______mldivide__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______mldivide__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______mldivide__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Matrix division from left.
-}
expDMatrix___mldivide__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___mldivide__ x = casADi__GenericExpression_CasADi__Matrix_double______mldivide__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______gt__" c_CasADi__GenericExpression_CasADi__Matrix_double______gt__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______gt__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______gt__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______gt__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>No need to have both < and >
-}
expDMatrix___gt__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___gt__ x = casADi__GenericExpression_CasADi__Matrix_double______gt__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______ge__" c_CasADi__GenericExpression_CasADi__Matrix_double______ge__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______ge__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______ge__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______ge__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>No need to have both <= and >=.
-}
expDMatrix___ge__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___ge__ x = casADi__GenericExpression_CasADi__Matrix_double______ge__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______truediv__" c_CasADi__GenericExpression_CasADi__Matrix_double______truediv__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______truediv__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______truediv__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______truediv__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Division (with future.division in effect)
-}
expDMatrix___truediv__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___truediv__ x = casADi__GenericExpression_CasADi__Matrix_double______truediv__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______radd__" c_CasADi__GenericExpression_CasADi__Matrix_double______radd__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______radd__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______radd__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______radd__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___radd__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___radd__ x = casADi__GenericExpression_CasADi__Matrix_double______radd__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rsub__" c_CasADi__GenericExpression_CasADi__Matrix_double______rsub__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rsub__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rsub__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rsub__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rsub__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rsub__ x = casADi__GenericExpression_CasADi__Matrix_double______rsub__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rmul__" c_CasADi__GenericExpression_CasADi__Matrix_double______rmul__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rmul__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rmul__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rmul__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rmul__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rmul__ x = casADi__GenericExpression_CasADi__Matrix_double______rmul__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rdiv__" c_CasADi__GenericExpression_CasADi__Matrix_double______rdiv__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rdiv__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rdiv__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rdiv__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rdiv__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rdiv__ x = casADi__GenericExpression_CasADi__Matrix_double______rdiv__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rlt__" c_CasADi__GenericExpression_CasADi__Matrix_double______rlt__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rlt__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rlt__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rlt__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rlt__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rlt__ x = casADi__GenericExpression_CasADi__Matrix_double______rlt__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rle__" c_CasADi__GenericExpression_CasADi__Matrix_double______rle__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rle__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rle__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rle__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rle__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rle__ x = casADi__GenericExpression_CasADi__Matrix_double______rle__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rgt__" c_CasADi__GenericExpression_CasADi__Matrix_double______rgt__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rgt__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rgt__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rgt__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rgt__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rgt__ x = casADi__GenericExpression_CasADi__Matrix_double______rgt__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rge__" c_CasADi__GenericExpression_CasADi__Matrix_double______rge__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rge__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rge__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rge__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rge__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rge__ x = casADi__GenericExpression_CasADi__Matrix_double______rge__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______req__" c_CasADi__GenericExpression_CasADi__Matrix_double______req__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______req__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______req__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______req__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___req__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___req__ x = casADi__GenericExpression_CasADi__Matrix_double______req__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rne__" c_CasADi__GenericExpression_CasADi__Matrix_double______rne__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rne__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rne__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rne__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rne__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rne__ x = casADi__GenericExpression_CasADi__Matrix_double______rne__ (castExpDMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_double______rtruediv__" c_CasADi__GenericExpression_CasADi__Matrix_double______rtruediv__
  :: Ptr ExpDMatrix' -> Ptr DMatrix' -> IO (Ptr DMatrix')
casADi__GenericExpression_CasADi__Matrix_double______rtruediv__
  :: ExpDMatrix -> DMatrix -> IO DMatrix
casADi__GenericExpression_CasADi__Matrix_double______rtruediv__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_double______rtruediv__ x0' x1' >>= wrapReturn

-- classy wrapper
expDMatrix___rtruediv__ :: ExpDMatrixClass a => a -> DMatrix -> IO DMatrix
expDMatrix___rtruediv__ x = casADi__GenericExpression_CasADi__Matrix_double______rtruediv__ (castExpDMatrix x)

