{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.ExpIMatrix
       (
         ExpIMatrix,
         ExpIMatrixClass(..),
         expIMatrix___ge__,
         expIMatrix___gt__,
         expIMatrix___mldivide__,
         expIMatrix___radd__,
         expIMatrix___rdiv__,
         expIMatrix___req__,
         expIMatrix___rge__,
         expIMatrix___rgt__,
         expIMatrix___rle__,
         expIMatrix___rlt__,
         expIMatrix___rmul__,
         expIMatrix___rne__,
         expIMatrix___rsub__,
         expIMatrix___rtruediv__,
         expIMatrix___truediv__,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______mldivide__" c_CasADi__GenericExpression_CasADi__Matrix_int______mldivide__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______mldivide__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______mldivide__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______mldivide__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Matrix division from left.
-}
expIMatrix___mldivide__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___mldivide__ x = casADi__GenericExpression_CasADi__Matrix_int______mldivide__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______gt__" c_CasADi__GenericExpression_CasADi__Matrix_int______gt__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______gt__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______gt__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______gt__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>No need to have both < and >
-}
expIMatrix___gt__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___gt__ x = casADi__GenericExpression_CasADi__Matrix_int______gt__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______ge__" c_CasADi__GenericExpression_CasADi__Matrix_int______ge__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______ge__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______ge__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______ge__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>No need to have both <= and >=.
-}
expIMatrix___ge__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___ge__ x = casADi__GenericExpression_CasADi__Matrix_int______ge__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______truediv__" c_CasADi__GenericExpression_CasADi__Matrix_int______truediv__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______truediv__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______truediv__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______truediv__ x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Division (with future.division in effect)
-}
expIMatrix___truediv__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___truediv__ x = casADi__GenericExpression_CasADi__Matrix_int______truediv__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______radd__" c_CasADi__GenericExpression_CasADi__Matrix_int______radd__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______radd__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______radd__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______radd__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___radd__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___radd__ x = casADi__GenericExpression_CasADi__Matrix_int______radd__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rsub__" c_CasADi__GenericExpression_CasADi__Matrix_int______rsub__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rsub__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rsub__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rsub__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rsub__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rsub__ x = casADi__GenericExpression_CasADi__Matrix_int______rsub__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rmul__" c_CasADi__GenericExpression_CasADi__Matrix_int______rmul__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rmul__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rmul__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rmul__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rmul__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rmul__ x = casADi__GenericExpression_CasADi__Matrix_int______rmul__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rdiv__" c_CasADi__GenericExpression_CasADi__Matrix_int______rdiv__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rdiv__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rdiv__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rdiv__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rdiv__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rdiv__ x = casADi__GenericExpression_CasADi__Matrix_int______rdiv__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rlt__" c_CasADi__GenericExpression_CasADi__Matrix_int______rlt__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rlt__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rlt__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rlt__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rlt__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rlt__ x = casADi__GenericExpression_CasADi__Matrix_int______rlt__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rle__" c_CasADi__GenericExpression_CasADi__Matrix_int______rle__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rle__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rle__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rle__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rle__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rle__ x = casADi__GenericExpression_CasADi__Matrix_int______rle__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rgt__" c_CasADi__GenericExpression_CasADi__Matrix_int______rgt__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rgt__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rgt__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rgt__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rgt__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rgt__ x = casADi__GenericExpression_CasADi__Matrix_int______rgt__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rge__" c_CasADi__GenericExpression_CasADi__Matrix_int______rge__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rge__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rge__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rge__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rge__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rge__ x = casADi__GenericExpression_CasADi__Matrix_int______rge__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______req__" c_CasADi__GenericExpression_CasADi__Matrix_int______req__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______req__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______req__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______req__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___req__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___req__ x = casADi__GenericExpression_CasADi__Matrix_int______req__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rne__" c_CasADi__GenericExpression_CasADi__Matrix_int______rne__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rne__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rne__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rne__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rne__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rne__ x = casADi__GenericExpression_CasADi__Matrix_int______rne__ (castExpIMatrix x)


-- direct wrapper
foreign import ccall unsafe "CasADi__GenericExpression_CasADi__Matrix_int______rtruediv__" c_CasADi__GenericExpression_CasADi__Matrix_int______rtruediv__
  :: Ptr ExpIMatrix' -> Ptr IMatrix' -> IO (Ptr IMatrix')
casADi__GenericExpression_CasADi__Matrix_int______rtruediv__
  :: ExpIMatrix -> IMatrix -> IO IMatrix
casADi__GenericExpression_CasADi__Matrix_int______rtruediv__ x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__GenericExpression_CasADi__Matrix_int______rtruediv__ x0' x1' >>= wrapReturn

-- classy wrapper
expIMatrix___rtruediv__ :: ExpIMatrixClass a => a -> IMatrix -> IO IMatrix
expIMatrix___rtruediv__ x = casADi__GenericExpression_CasADi__Matrix_int______rtruediv__ (castExpIMatrix x)

