{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.KinsolSolver
       (
         KinsolSolver,
         KinsolSolverClass(..),
         kinsolSolver,
         kinsolSolver',
         kinsolSolver'',
         kinsolSolver''',
         kinsolSolver_checkNode,
         kinsolSolver_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show KinsolSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__KinsolSolver__checkNode" c_CasADi__KinsolSolver__checkNode
  :: Ptr KinsolSolver' -> IO CInt
casADi__KinsolSolver__checkNode
  :: KinsolSolver -> IO Bool
casADi__KinsolSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__KinsolSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
kinsolSolver_checkNode :: KinsolSolverClass a => a -> IO Bool
kinsolSolver_checkNode x = casADi__KinsolSolver__checkNode (castKinsolSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__KinsolSolver__creator" c_CasADi__KinsolSolver__creator
  :: Ptr Function' -> Ptr Function' -> Ptr LinearSolver' -> IO (Ptr ImplicitFunction')
casADi__KinsolSolver__creator
  :: Function -> Function -> LinearSolver -> IO ImplicitFunction
casADi__KinsolSolver__creator x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__KinsolSolver__creator x0' x1' x2' >>= wrapReturn

-- classy wrapper
kinsolSolver_creator :: Function -> Function -> LinearSolver -> IO ImplicitFunction
kinsolSolver_creator = casADi__KinsolSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__KinsolSolver__KinsolSolver" c_CasADi__KinsolSolver__KinsolSolver
  :: IO (Ptr KinsolSolver')
casADi__KinsolSolver__KinsolSolver
  :: IO KinsolSolver
casADi__KinsolSolver__KinsolSolver  =
  c_CasADi__KinsolSolver__KinsolSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::KinsolSolver::KinsolSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::KinsolSolver::KinsolSolver(const Function &f, const Function &jac=Function(), const LinearSolver &linsol=LinearSolver())
>------------------------------------------------------------------------
>
>Create an KINSOL instance.
>
>Parameters:
>-----------
>
>f:   Function mapping from (n+1) inputs to 1 output
-}
kinsolSolver :: IO KinsolSolver
kinsolSolver = casADi__KinsolSolver__KinsolSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__KinsolSolver__KinsolSolver_TIC" c_CasADi__KinsolSolver__KinsolSolver_TIC
  :: Ptr Function' -> Ptr Function' -> Ptr LinearSolver' -> IO (Ptr KinsolSolver')
casADi__KinsolSolver__KinsolSolver'
  :: Function -> Function -> LinearSolver -> IO KinsolSolver
casADi__KinsolSolver__KinsolSolver' x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__KinsolSolver__KinsolSolver_TIC x0' x1' x2' >>= wrapReturn

-- classy wrapper
kinsolSolver' :: Function -> Function -> LinearSolver -> IO KinsolSolver
kinsolSolver' = casADi__KinsolSolver__KinsolSolver'


-- direct wrapper
foreign import ccall unsafe "CasADi__KinsolSolver__KinsolSolver_TIC_TIC" c_CasADi__KinsolSolver__KinsolSolver_TIC_TIC
  :: Ptr Function' -> Ptr Function' -> IO (Ptr KinsolSolver')
casADi__KinsolSolver__KinsolSolver''
  :: Function -> Function -> IO KinsolSolver
casADi__KinsolSolver__KinsolSolver'' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__KinsolSolver__KinsolSolver_TIC_TIC x0' x1' >>= wrapReturn

-- classy wrapper
kinsolSolver'' :: Function -> Function -> IO KinsolSolver
kinsolSolver'' = casADi__KinsolSolver__KinsolSolver''


-- direct wrapper
foreign import ccall unsafe "CasADi__KinsolSolver__KinsolSolver_TIC_TIC_TIC" c_CasADi__KinsolSolver__KinsolSolver_TIC_TIC_TIC
  :: Ptr Function' -> IO (Ptr KinsolSolver')
casADi__KinsolSolver__KinsolSolver'''
  :: Function -> IO KinsolSolver
casADi__KinsolSolver__KinsolSolver''' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__KinsolSolver__KinsolSolver_TIC_TIC_TIC x0' >>= wrapReturn

-- classy wrapper
kinsolSolver''' :: Function -> IO KinsolSolver
kinsolSolver''' = casADi__KinsolSolver__KinsolSolver'''

