{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.LPSolver
       (
         LPSolver,
         LPSolverClass(..),
         lpSolver,
         lpSolver_checkNode,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show LPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__LPSolver__checkNode" c_CasADi__LPSolver__checkNode
  :: Ptr LPSolver' -> IO CInt
casADi__LPSolver__checkNode
  :: LPSolver -> IO Bool
casADi__LPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__LPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
lpSolver_checkNode :: LPSolverClass a => a -> IO Bool
lpSolver_checkNode x = casADi__LPSolver__checkNode (castLPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__LPSolver__LPSolver" c_CasADi__LPSolver__LPSolver
  :: IO (Ptr LPSolver')
casADi__LPSolver__LPSolver
  :: IO LPSolver
casADi__LPSolver__LPSolver  =
  c_CasADi__LPSolver__LPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
lpSolver :: IO LPSolver
lpSolver = casADi__LPSolver__LPSolver

