{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.NLPImplicitSolver
       (
         NLPImplicitSolver,
         NLPImplicitSolverClass(..),
         nlpImplicitSolver,
         nlpImplicitSolver',
         nlpImplicitSolver'',
         nlpImplicitSolver''',
         nlpImplicitSolver_checkNode,
         nlpImplicitSolver_creator,
         nlpImplicitSolver_getNLPSolver,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show NLPImplicitSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__NLPImplicitSolver__checkNode" c_CasADi__NLPImplicitSolver__checkNode
  :: Ptr NLPImplicitSolver' -> IO CInt
casADi__NLPImplicitSolver__checkNode
  :: NLPImplicitSolver -> IO Bool
casADi__NLPImplicitSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NLPImplicitSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
nlpImplicitSolver_checkNode :: NLPImplicitSolverClass a => a -> IO Bool
nlpImplicitSolver_checkNode x = casADi__NLPImplicitSolver__checkNode (castNLPImplicitSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPImplicitSolver__getNLPSolver" c_CasADi__NLPImplicitSolver__getNLPSolver
  :: Ptr NLPImplicitSolver' -> IO (Ptr NLPSolver')
casADi__NLPImplicitSolver__getNLPSolver
  :: NLPImplicitSolver -> IO NLPSolver
casADi__NLPImplicitSolver__getNLPSolver x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NLPImplicitSolver__getNLPSolver x0' >>= wrapReturn

-- classy wrapper
{-|
>Access NLP solver.
-}
nlpImplicitSolver_getNLPSolver :: NLPImplicitSolverClass a => a -> IO NLPSolver
nlpImplicitSolver_getNLPSolver x = casADi__NLPImplicitSolver__getNLPSolver (castNLPImplicitSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPImplicitSolver__creator" c_CasADi__NLPImplicitSolver__creator
  :: Ptr Function' -> Ptr Function' -> Ptr LinearSolver' -> IO (Ptr ImplicitFunction')
casADi__NLPImplicitSolver__creator
  :: Function -> Function -> LinearSolver -> IO ImplicitFunction
casADi__NLPImplicitSolver__creator x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__NLPImplicitSolver__creator x0' x1' x2' >>= wrapReturn

-- classy wrapper
nlpImplicitSolver_creator :: Function -> Function -> LinearSolver -> IO ImplicitFunction
nlpImplicitSolver_creator = casADi__NLPImplicitSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPImplicitSolver__NLPImplicitSolver" c_CasADi__NLPImplicitSolver__NLPImplicitSolver
  :: IO (Ptr NLPImplicitSolver')
casADi__NLPImplicitSolver__NLPImplicitSolver
  :: IO NLPImplicitSolver
casADi__NLPImplicitSolver__NLPImplicitSolver  =
  c_CasADi__NLPImplicitSolver__NLPImplicitSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::NLPImplicitSolver::NLPImplicitSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::NLPImplicitSolver::NLPImplicitSolver(const Function &f, const Function &jac=Function(), const LinearSolver &linsol=LinearSolver())
>------------------------------------------------------------------------
>
>Create a new solver instance.
-}
nlpImplicitSolver :: IO NLPImplicitSolver
nlpImplicitSolver = casADi__NLPImplicitSolver__NLPImplicitSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC" c_CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC
  :: Ptr Function' -> Ptr Function' -> Ptr LinearSolver' -> IO (Ptr NLPImplicitSolver')
casADi__NLPImplicitSolver__NLPImplicitSolver'
  :: Function -> Function -> LinearSolver -> IO NLPImplicitSolver
casADi__NLPImplicitSolver__NLPImplicitSolver' x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC x0' x1' x2' >>= wrapReturn

-- classy wrapper
nlpImplicitSolver' :: Function -> Function -> LinearSolver -> IO NLPImplicitSolver
nlpImplicitSolver' = casADi__NLPImplicitSolver__NLPImplicitSolver'


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC_TIC" c_CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC_TIC
  :: Ptr Function' -> Ptr Function' -> IO (Ptr NLPImplicitSolver')
casADi__NLPImplicitSolver__NLPImplicitSolver''
  :: Function -> Function -> IO NLPImplicitSolver
casADi__NLPImplicitSolver__NLPImplicitSolver'' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC_TIC x0' x1' >>= wrapReturn

-- classy wrapper
nlpImplicitSolver'' :: Function -> Function -> IO NLPImplicitSolver
nlpImplicitSolver'' = casADi__NLPImplicitSolver__NLPImplicitSolver''


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC_TIC_TIC" c_CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC_TIC_TIC
  :: Ptr Function' -> IO (Ptr NLPImplicitSolver')
casADi__NLPImplicitSolver__NLPImplicitSolver'''
  :: Function -> IO NLPImplicitSolver
casADi__NLPImplicitSolver__NLPImplicitSolver''' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NLPImplicitSolver__NLPImplicitSolver_TIC_TIC_TIC x0' >>= wrapReturn

-- classy wrapper
nlpImplicitSolver''' :: Function -> IO NLPImplicitSolver
nlpImplicitSolver''' = casADi__NLPImplicitSolver__NLPImplicitSolver'''

