{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.NLPQPSolver
       (
         NLPQPSolver,
         NLPQPSolverClass(..),
         nlpqpSolver,
         nlpqpSolver',
         nlpqpSolver_checkNode,
         nlpqpSolver_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show NLPQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__NLPQPSolver__checkNode" c_CasADi__NLPQPSolver__checkNode
  :: Ptr NLPQPSolver' -> IO CInt
casADi__NLPQPSolver__checkNode
  :: NLPQPSolver -> IO Bool
casADi__NLPQPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NLPQPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>>  bool CasADi::NLPQPSolver::checkNode() const 
>------------------------------------------------------------------------
>
>Check if the node is pointing to the right type of object.
>
>>  virtual bool CasADi::NLPQPSolver::checkNode() const 
>------------------------------------------------------------------------
>[INTERNAL] 
>Check if the node is pointing to the right type of object.
-}
nlpqpSolver_checkNode :: NLPQPSolverClass a => a -> IO Bool
nlpqpSolver_checkNode x = casADi__NLPQPSolver__checkNode (castNLPQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPQPSolver__creator" c_CasADi__NLPQPSolver__creator
  :: Ptr QPStructure' -> IO (Ptr QPSolver')
casADi__NLPQPSolver__creator
  :: QPStructure -> IO QPSolver
casADi__NLPQPSolver__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NLPQPSolver__creator x0' >>= wrapReturn

-- classy wrapper
nlpqpSolver_creator :: QPStructure -> IO QPSolver
nlpqpSolver_creator = casADi__NLPQPSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPQPSolver__NLPQPSolver" c_CasADi__NLPQPSolver__NLPQPSolver
  :: IO (Ptr NLPQPSolver')
casADi__NLPQPSolver__NLPQPSolver
  :: IO NLPQPSolver
casADi__NLPQPSolver__NLPQPSolver  =
  c_CasADi__NLPQPSolver__NLPQPSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::NLPQPSolver::NLPQPSolver()
>
>>  CasADi::NLPQPSolver::NLPQPSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::NLPQPSolver::NLPQPSolver(const QPStructure &st)
>
>>  CasADi::NLPQPSolver::NLPQPSolver(const QPStructure &st)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>st:  Problem structure
-}
nlpqpSolver :: IO NLPQPSolver
nlpqpSolver = casADi__NLPQPSolver__NLPQPSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__NLPQPSolver__NLPQPSolver_TIC" c_CasADi__NLPQPSolver__NLPQPSolver_TIC
  :: Ptr QPStructure' -> IO (Ptr NLPQPSolver')
casADi__NLPQPSolver__NLPQPSolver'
  :: QPStructure -> IO NLPQPSolver
casADi__NLPQPSolver__NLPQPSolver' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NLPQPSolver__NLPQPSolver_TIC x0' >>= wrapReturn

-- classy wrapper
nlpqpSolver' :: QPStructure -> IO NLPQPSolver
nlpqpSolver' = casADi__NLPQPSolver__NLPQPSolver'

