{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.NewtonImplicitSolver
       (
         NewtonImplicitSolver,
         NewtonImplicitSolverClass(..),
         newtonImplicitSolver,
         newtonImplicitSolver',
         newtonImplicitSolver'',
         newtonImplicitSolver''',
         newtonImplicitSolver_checkNode,
         newtonImplicitSolver_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show NewtonImplicitSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__NewtonImplicitSolver__checkNode" c_CasADi__NewtonImplicitSolver__checkNode
  :: Ptr NewtonImplicitSolver' -> IO CInt
casADi__NewtonImplicitSolver__checkNode
  :: NewtonImplicitSolver -> IO Bool
casADi__NewtonImplicitSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NewtonImplicitSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
newtonImplicitSolver_checkNode :: NewtonImplicitSolverClass a => a -> IO Bool
newtonImplicitSolver_checkNode x = casADi__NewtonImplicitSolver__checkNode (castNewtonImplicitSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__NewtonImplicitSolver__creator" c_CasADi__NewtonImplicitSolver__creator
  :: Ptr Function' -> Ptr Function' -> Ptr LinearSolver' -> IO (Ptr ImplicitFunction')
casADi__NewtonImplicitSolver__creator
  :: Function -> Function -> LinearSolver -> IO ImplicitFunction
casADi__NewtonImplicitSolver__creator x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__NewtonImplicitSolver__creator x0' x1' x2' >>= wrapReturn

-- classy wrapper
newtonImplicitSolver_creator :: Function -> Function -> LinearSolver -> IO ImplicitFunction
newtonImplicitSolver_creator = casADi__NewtonImplicitSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__NewtonImplicitSolver__NewtonImplicitSolver" c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver
  :: IO (Ptr NewtonImplicitSolver')
casADi__NewtonImplicitSolver__NewtonImplicitSolver
  :: IO NewtonImplicitSolver
casADi__NewtonImplicitSolver__NewtonImplicitSolver  =
  c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::NewtonImplicitSolver::NewtonImplicitSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::NewtonImplicitSolver::NewtonImplicitSolver(const Function &f, const Function &jac=Function(), const LinearSolver &linsol=LinearSolver())
>------------------------------------------------------------------------
>
>Create a solver instance.
-}
newtonImplicitSolver :: IO NewtonImplicitSolver
newtonImplicitSolver = casADi__NewtonImplicitSolver__NewtonImplicitSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC" c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC
  :: Ptr Function' -> Ptr Function' -> Ptr LinearSolver' -> IO (Ptr NewtonImplicitSolver')
casADi__NewtonImplicitSolver__NewtonImplicitSolver'
  :: Function -> Function -> LinearSolver -> IO NewtonImplicitSolver
casADi__NewtonImplicitSolver__NewtonImplicitSolver' x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC x0' x1' x2' >>= wrapReturn

-- classy wrapper
newtonImplicitSolver' :: Function -> Function -> LinearSolver -> IO NewtonImplicitSolver
newtonImplicitSolver' = casADi__NewtonImplicitSolver__NewtonImplicitSolver'


-- direct wrapper
foreign import ccall unsafe "CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC_TIC" c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC_TIC
  :: Ptr Function' -> Ptr Function' -> IO (Ptr NewtonImplicitSolver')
casADi__NewtonImplicitSolver__NewtonImplicitSolver''
  :: Function -> Function -> IO NewtonImplicitSolver
casADi__NewtonImplicitSolver__NewtonImplicitSolver'' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC_TIC x0' x1' >>= wrapReturn

-- classy wrapper
newtonImplicitSolver'' :: Function -> Function -> IO NewtonImplicitSolver
newtonImplicitSolver'' = casADi__NewtonImplicitSolver__NewtonImplicitSolver''


-- direct wrapper
foreign import ccall unsafe "CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC_TIC_TIC" c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC_TIC_TIC
  :: Ptr Function' -> IO (Ptr NewtonImplicitSolver')
casADi__NewtonImplicitSolver__NewtonImplicitSolver'''
  :: Function -> IO NewtonImplicitSolver
casADi__NewtonImplicitSolver__NewtonImplicitSolver''' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__NewtonImplicitSolver__NewtonImplicitSolver_TIC_TIC_TIC x0' >>= wrapReturn

-- classy wrapper
newtonImplicitSolver''' :: Function -> IO NewtonImplicitSolver
newtonImplicitSolver''' = casADi__NewtonImplicitSolver__NewtonImplicitSolver'''

