{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.Nullspace
       (
         Nullspace,
         NullspaceClass(..),
         nullspace,
         nullspace_checkNode,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show Nullspace where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__Nullspace__checkNode" c_CasADi__Nullspace__checkNode
  :: Ptr Nullspace' -> IO CInt
casADi__Nullspace__checkNode
  :: Nullspace -> IO Bool
casADi__Nullspace__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__Nullspace__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
nullspace_checkNode :: NullspaceClass a => a -> IO Bool
nullspace_checkNode x = casADi__Nullspace__checkNode (castNullspace x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Nullspace__Nullspace" c_CasADi__Nullspace__Nullspace
  :: IO (Ptr Nullspace')
casADi__Nullspace__Nullspace
  :: IO Nullspace
casADi__Nullspace__Nullspace  =
  c_CasADi__Nullspace__Nullspace  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
nullspace :: IO Nullspace
nullspace = casADi__Nullspace__Nullspace

