{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.OCPSolver
       (
         OCPSolver,
         OCPSolverClass(..),
         ocpSolver,
         ocpSolver_getCfcn,
         ocpSolver_getFfcn,
         ocpSolver_getMfcn,
         ocpSolver_getRfcn,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show OCPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__OCPSolver__getFfcn" c_CasADi__OCPSolver__getFfcn
  :: Ptr OCPSolver' -> IO (Ptr Function')
casADi__OCPSolver__getFfcn
  :: OCPSolver -> IO Function
casADi__OCPSolver__getFfcn x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OCPSolver__getFfcn x0' >>= wrapReturn

-- classy wrapper
ocpSolver_getFfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getFfcn x = casADi__OCPSolver__getFfcn (castOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OCPSolver__getMfcn" c_CasADi__OCPSolver__getMfcn
  :: Ptr OCPSolver' -> IO (Ptr Function')
casADi__OCPSolver__getMfcn
  :: OCPSolver -> IO Function
casADi__OCPSolver__getMfcn x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OCPSolver__getMfcn x0' >>= wrapReturn

-- classy wrapper
ocpSolver_getMfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getMfcn x = casADi__OCPSolver__getMfcn (castOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OCPSolver__getCfcn" c_CasADi__OCPSolver__getCfcn
  :: Ptr OCPSolver' -> IO (Ptr Function')
casADi__OCPSolver__getCfcn
  :: OCPSolver -> IO Function
casADi__OCPSolver__getCfcn x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OCPSolver__getCfcn x0' >>= wrapReturn

-- classy wrapper
ocpSolver_getCfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getCfcn x = casADi__OCPSolver__getCfcn (castOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OCPSolver__getRfcn" c_CasADi__OCPSolver__getRfcn
  :: Ptr OCPSolver' -> IO (Ptr Function')
casADi__OCPSolver__getRfcn
  :: OCPSolver -> IO Function
casADi__OCPSolver__getRfcn x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OCPSolver__getRfcn x0' >>= wrapReturn

-- classy wrapper
ocpSolver_getRfcn :: OCPSolverClass a => a -> IO Function
ocpSolver_getRfcn x = casADi__OCPSolver__getRfcn (castOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OCPSolver__OCPSolver" c_CasADi__OCPSolver__OCPSolver
  :: IO (Ptr OCPSolver')
casADi__OCPSolver__OCPSolver
  :: IO OCPSolver
casADi__OCPSolver__OCPSolver  =
  c_CasADi__OCPSolver__OCPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
ocpSolver :: IO OCPSolver
ocpSolver = casADi__OCPSolver__OCPSolver

