{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.OldCollocationIntegrator
       (
         OldCollocationIntegrator,
         OldCollocationIntegratorClass(..),
         oldCollocationIntegrator,
         oldCollocationIntegrator',
         oldCollocationIntegrator'',
         oldCollocationIntegrator_checkNode,
         oldCollocationIntegrator_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show OldCollocationIntegrator where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__OldCollocationIntegrator__checkNode" c_CasADi__OldCollocationIntegrator__checkNode
  :: Ptr OldCollocationIntegrator' -> IO CInt
casADi__OldCollocationIntegrator__checkNode
  :: OldCollocationIntegrator -> IO Bool
casADi__OldCollocationIntegrator__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OldCollocationIntegrator__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
oldCollocationIntegrator_checkNode :: OldCollocationIntegratorClass a => a -> IO Bool
oldCollocationIntegrator_checkNode x = casADi__OldCollocationIntegrator__checkNode (castOldCollocationIntegrator x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OldCollocationIntegrator__creator" c_CasADi__OldCollocationIntegrator__creator
  :: Ptr Function' -> Ptr Function' -> IO (Ptr Integrator')
casADi__OldCollocationIntegrator__creator
  :: Function -> Function -> IO Integrator
casADi__OldCollocationIntegrator__creator x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OldCollocationIntegrator__creator x0' x1' >>= wrapReturn

-- classy wrapper
oldCollocationIntegrator_creator :: Function -> Function -> IO Integrator
oldCollocationIntegrator_creator = casADi__OldCollocationIntegrator__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__OldCollocationIntegrator__OldCollocationIntegrator" c_CasADi__OldCollocationIntegrator__OldCollocationIntegrator
  :: IO (Ptr OldCollocationIntegrator')
casADi__OldCollocationIntegrator__OldCollocationIntegrator
  :: IO OldCollocationIntegrator
casADi__OldCollocationIntegrator__OldCollocationIntegrator  =
  c_CasADi__OldCollocationIntegrator__OldCollocationIntegrator  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::OldCollocationIntegrator::OldCollocationIntegrator()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::OldCollocationIntegrator::OldCollocationIntegrator(const Function &f, const Function &g=Function())
>------------------------------------------------------------------------
>
>Create an integrator for explicit ODEs.
>
>Parameters:
>-----------
>
>f:  dynamical system
>
>>Input scheme: CasADi::DAEInput (DAE_NUM_IN = 5) [daeIn]
>+-----------+-------+----------------------------+
>| Full name | Short |        Description         |
>+===========+=======+============================+
>| DAE_X     | x     | Differential state .       |
>+-----------+-------+----------------------------+
>| DAE_Z     | z     | Algebraic state .          |
>+-----------+-------+----------------------------+
>| DAE_P     | p     | Parameter .                |
>+-----------+-------+----------------------------+
>| DAE_T     | t     | Explicit time dependence . |
>+-----------+-------+----------------------------+
>
>>Output scheme: CasADi::DAEOutput (DAE_NUM_OUT = 4) [daeOut]
>+-----------+-------+--------------------------------------------+
>| Full name | Short |                Description                 |
>+===========+=======+============================================+
>| DAE_ODE   | ode   | Right hand side of the implicit ODE .      |
>+-----------+-------+--------------------------------------------+
>| DAE_ALG   | alg   | Right hand side of algebraic equations .   |
>+-----------+-------+--------------------------------------------+
>| DAE_QUAD  | quad  | Right hand side of quadratures equations . |
>+-----------+-------+--------------------------------------------+
>
>Parameters:
>-----------
>
>g:  backwards system
>
>>Input scheme: CasADi::RDAEInput (RDAE_NUM_IN = 8) [rdaeIn]
>+-----------+-------+-------------------------------+
>| Full name | Short |          Description          |
>+===========+=======+===============================+
>| RDAE_RX   | rx    | Backward differential state . |
>+-----------+-------+-------------------------------+
>| RDAE_RZ   | rz    | Backward algebraic state .    |
>+-----------+-------+-------------------------------+
>| RDAE_RP   | rp    | Backward parameter vector .   |
>+-----------+-------+-------------------------------+
>| RDAE_X    | x     | Forward differential state .  |
>+-----------+-------+-------------------------------+
>| RDAE_Z    | z     | Forward algebraic state .     |
>+-----------+-------+-------------------------------+
>| RDAE_P    | p     | Parameter vector .            |
>+-----------+-------+-------------------------------+
>| RDAE_T    | t     | Explicit time dependence .    |
>+-----------+-------+-------------------------------+
>
>>Output scheme: CasADi::RDAEOutput (RDAE_NUM_OUT = 4) [rdaeOut]
>+-----------+-------+-------------------------------------------+
>| Full name | Short |                Description                |
>+===========+=======+===========================================+
>| RDAE_ODE  | ode   | Right hand side of ODE. .                 |
>+-----------+-------+-------------------------------------------+
>| RDAE_ALG  | alg   | Right hand side of algebraic equations. . |
>+-----------+-------+-------------------------------------------+
>| RDAE_QUAD | quad  | Right hand side of quadratures. .         |
>+-----------+-------+-------------------------------------------+
-}
oldCollocationIntegrator :: IO OldCollocationIntegrator
oldCollocationIntegrator = casADi__OldCollocationIntegrator__OldCollocationIntegrator


-- direct wrapper
foreign import ccall unsafe "CasADi__OldCollocationIntegrator__OldCollocationIntegrator_TIC" c_CasADi__OldCollocationIntegrator__OldCollocationIntegrator_TIC
  :: Ptr Function' -> Ptr Function' -> IO (Ptr OldCollocationIntegrator')
casADi__OldCollocationIntegrator__OldCollocationIntegrator'
  :: Function -> Function -> IO OldCollocationIntegrator
casADi__OldCollocationIntegrator__OldCollocationIntegrator' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OldCollocationIntegrator__OldCollocationIntegrator_TIC x0' x1' >>= wrapReturn

-- classy wrapper
oldCollocationIntegrator' :: Function -> Function -> IO OldCollocationIntegrator
oldCollocationIntegrator' = casADi__OldCollocationIntegrator__OldCollocationIntegrator'


-- direct wrapper
foreign import ccall unsafe "CasADi__OldCollocationIntegrator__OldCollocationIntegrator_TIC_TIC" c_CasADi__OldCollocationIntegrator__OldCollocationIntegrator_TIC_TIC
  :: Ptr Function' -> IO (Ptr OldCollocationIntegrator')
casADi__OldCollocationIntegrator__OldCollocationIntegrator''
  :: Function -> IO OldCollocationIntegrator
casADi__OldCollocationIntegrator__OldCollocationIntegrator'' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OldCollocationIntegrator__OldCollocationIntegrator_TIC_TIC x0' >>= wrapReturn

-- classy wrapper
oldCollocationIntegrator'' :: Function -> IO OldCollocationIntegrator
oldCollocationIntegrator'' = casADi__OldCollocationIntegrator__OldCollocationIntegrator''

