{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.QCQPQPSolver
       (
         QCQPQPSolver,
         QCQPQPSolverClass(..),
         qcqpqpSolver,
         qcqpqpSolver',
         qcqpqpSolver_checkNode,
         qcqpqpSolver_creator,
         qcqpqpSolver_getSolver,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show QCQPQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPQPSolver__checkNode" c_CasADi__QCQPQPSolver__checkNode
  :: Ptr QCQPQPSolver' -> IO CInt
casADi__QCQPQPSolver__checkNode
  :: QCQPQPSolver -> IO Bool
casADi__QCQPQPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QCQPQPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
qcqpqpSolver_checkNode :: QCQPQPSolverClass a => a -> IO Bool
qcqpqpSolver_checkNode x = casADi__QCQPQPSolver__checkNode (castQCQPQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPQPSolver__creator" c_CasADi__QCQPQPSolver__creator
  :: Ptr QPStructure' -> IO (Ptr QPSolver')
casADi__QCQPQPSolver__creator
  :: QPStructure -> IO QPSolver
casADi__QCQPQPSolver__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QCQPQPSolver__creator x0' >>= wrapReturn

-- classy wrapper
qcqpqpSolver_creator :: QPStructure -> IO QPSolver
qcqpqpSolver_creator = casADi__QCQPQPSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPQPSolver__getSolver" c_CasADi__QCQPQPSolver__getSolver
  :: Ptr QCQPQPSolver' -> IO (Ptr QCQPSolver')
casADi__QCQPQPSolver__getSolver
  :: QCQPQPSolver -> IO QCQPSolver
casADi__QCQPQPSolver__getSolver x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QCQPQPSolver__getSolver x0' >>= wrapReturn

-- classy wrapper
{-|
>Access underlying QCQP solver.
-}
qcqpqpSolver_getSolver :: QCQPQPSolverClass a => a -> IO QCQPSolver
qcqpqpSolver_getSolver x = casADi__QCQPQPSolver__getSolver (castQCQPQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPQPSolver__QCQPQPSolver" c_CasADi__QCQPQPSolver__QCQPQPSolver
  :: IO (Ptr QCQPQPSolver')
casADi__QCQPQPSolver__QCQPQPSolver
  :: IO QCQPQPSolver
casADi__QCQPQPSolver__QCQPQPSolver  =
  c_CasADi__QCQPQPSolver__QCQPQPSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::QCQPQPSolver::QCQPQPSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::QCQPQPSolver::QCQPQPSolver(const QPStructure &st)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>st:  Problem structure
-}
qcqpqpSolver :: IO QCQPQPSolver
qcqpqpSolver = casADi__QCQPQPSolver__QCQPQPSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPQPSolver__QCQPQPSolver_TIC" c_CasADi__QCQPQPSolver__QCQPQPSolver_TIC
  :: Ptr QPStructure' -> IO (Ptr QCQPQPSolver')
casADi__QCQPQPSolver__QCQPQPSolver'
  :: QPStructure -> IO QCQPQPSolver
casADi__QCQPQPSolver__QCQPQPSolver' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QCQPQPSolver__QCQPQPSolver_TIC x0' >>= wrapReturn

-- classy wrapper
qcqpqpSolver' :: QPStructure -> IO QCQPQPSolver
qcqpqpSolver' = casADi__QCQPQPSolver__QCQPQPSolver'

