{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.QCQPSolver
       (
         QCQPSolver,
         QCQPSolverClass(..),
         qcqpSolver,
         qcqpSolver_checkNode,
         qcqpSolver_setQPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show QCQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPSolver__checkNode" c_CasADi__QCQPSolver__checkNode
  :: Ptr QCQPSolver' -> IO CInt
casADi__QCQPSolver__checkNode
  :: QCQPSolver -> IO Bool
casADi__QCQPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QCQPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
qcqpSolver_checkNode :: QCQPSolverClass a => a -> IO Bool
qcqpSolver_checkNode x = casADi__QCQPSolver__checkNode (castQCQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPSolver__setQPOptions" c_CasADi__QCQPSolver__setQPOptions
  :: Ptr QCQPSolver' -> IO ()
casADi__QCQPSolver__setQPOptions
  :: QCQPSolver -> IO ()
casADi__QCQPSolver__setQPOptions x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QCQPSolver__setQPOptions x0' >>= wrapReturn

-- classy wrapper
{-|
>Set options that make the QP solver more suitable for solving LPs.
-}
qcqpSolver_setQPOptions :: QCQPSolverClass a => a -> IO ()
qcqpSolver_setQPOptions x = casADi__QCQPSolver__setQPOptions (castQCQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QCQPSolver__QCQPSolver" c_CasADi__QCQPSolver__QCQPSolver
  :: IO (Ptr QCQPSolver')
casADi__QCQPSolver__QCQPSolver
  :: IO QCQPSolver
casADi__QCQPSolver__QCQPSolver  =
  c_CasADi__QCQPSolver__QCQPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
qcqpSolver :: IO QCQPSolver
qcqpSolver = casADi__QCQPSolver__QCQPSolver

