{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.QPLPSolver
       (
         QPLPSolver,
         QPLPSolverClass(..),
         qplpSolver,
         qplpSolver',
         qplpSolver_checkNode,
         qplpSolver_creator,
         qplpSolver_getSolver,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show QPLPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__QPLPSolver__checkNode" c_CasADi__QPLPSolver__checkNode
  :: Ptr QPLPSolver' -> IO CInt
casADi__QPLPSolver__checkNode
  :: QPLPSolver -> IO Bool
casADi__QPLPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPLPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
qplpSolver_checkNode :: QPLPSolverClass a => a -> IO Bool
qplpSolver_checkNode x = casADi__QPLPSolver__checkNode (castQPLPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QPLPSolver__creator" c_CasADi__QPLPSolver__creator
  :: Ptr LPStructure' -> IO (Ptr LPSolver')
casADi__QPLPSolver__creator
  :: LPStructure -> IO LPSolver
casADi__QPLPSolver__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPLPSolver__creator x0' >>= wrapReturn

-- classy wrapper
qplpSolver_creator :: LPStructure -> IO LPSolver
qplpSolver_creator = casADi__QPLPSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__QPLPSolver__getSolver" c_CasADi__QPLPSolver__getSolver
  :: Ptr QPLPSolver' -> IO (Ptr QPSolver')
casADi__QPLPSolver__getSolver
  :: QPLPSolver -> IO QPSolver
casADi__QPLPSolver__getSolver x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPLPSolver__getSolver x0' >>= wrapReturn

-- classy wrapper
{-|
>Access underlying QP solver.
-}
qplpSolver_getSolver :: QPLPSolverClass a => a -> IO QPSolver
qplpSolver_getSolver x = casADi__QPLPSolver__getSolver (castQPLPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QPLPSolver__QPLPSolver" c_CasADi__QPLPSolver__QPLPSolver
  :: IO (Ptr QPLPSolver')
casADi__QPLPSolver__QPLPSolver
  :: IO QPLPSolver
casADi__QPLPSolver__QPLPSolver  =
  c_CasADi__QPLPSolver__QPLPSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::QPLPSolver::QPLPSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::QPLPSolver::QPLPSolver(const LPStructure &st)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>st:  Problem structure
-}
qplpSolver :: IO QPLPSolver
qplpSolver = casADi__QPLPSolver__QPLPSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__QPLPSolver__QPLPSolver_TIC" c_CasADi__QPLPSolver__QPLPSolver_TIC
  :: Ptr LPStructure' -> IO (Ptr QPLPSolver')
casADi__QPLPSolver__QPLPSolver'
  :: LPStructure -> IO QPLPSolver
casADi__QPLPSolver__QPLPSolver' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPLPSolver__QPLPSolver_TIC x0' >>= wrapReturn

-- classy wrapper
qplpSolver' :: LPStructure -> IO QPLPSolver
qplpSolver' = casADi__QPLPSolver__QPLPSolver'

