{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.QPSolver
       (
         QPSolver,
         QPSolverClass(..),
         qpSolver,
         qpSolver_checkNode,
         qpSolver_generateNativeCode,
         qpSolver_setLPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show QPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__QPSolver__checkNode" c_CasADi__QPSolver__checkNode
  :: Ptr QPSolver' -> IO CInt
casADi__QPSolver__checkNode
  :: QPSolver -> IO Bool
casADi__QPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
qpSolver_checkNode :: QPSolverClass a => a -> IO Bool
qpSolver_checkNode x = casADi__QPSolver__checkNode (castQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QPSolver__setLPOptions" c_CasADi__QPSolver__setLPOptions
  :: Ptr QPSolver' -> IO ()
casADi__QPSolver__setLPOptions
  :: QPSolver -> IO ()
casADi__QPSolver__setLPOptions x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPSolver__setLPOptions x0' >>= wrapReturn

-- classy wrapper
{-|
>Set options that make the QP solver more suitable for solving LPs.
-}
qpSolver_setLPOptions :: QPSolverClass a => a -> IO ()
qpSolver_setLPOptions x = casADi__QPSolver__setLPOptions (castQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QPSolver__generateNativeCode" c_CasADi__QPSolver__generateNativeCode
  :: Ptr QPSolver' -> Ptr StdString' -> IO ()
casADi__QPSolver__generateNativeCode
  :: QPSolver -> String -> IO ()
casADi__QPSolver__generateNativeCode x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__QPSolver__generateNativeCode x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Generate native code in the interfaced language for debugging
-}
qpSolver_generateNativeCode :: QPSolverClass a => a -> String -> IO ()
qpSolver_generateNativeCode x = casADi__QPSolver__generateNativeCode (castQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QPSolver__QPSolver" c_CasADi__QPSolver__QPSolver
  :: IO (Ptr QPSolver')
casADi__QPSolver__QPSolver
  :: IO QPSolver
casADi__QPSolver__QPSolver  =
  c_CasADi__QPSolver__QPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
qpSolver :: IO QPSolver
qpSolver = casADi__QPSolver__QPSolver

