{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.QPStabilizer
       (
         QPStabilizer,
         QPStabilizerClass(..),
         qpStabilizer,
         qpStabilizer',
         qpStabilizer_checkNode,
         qpStabilizer_creator,
         qpStabilizer_getSolver,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show QPStabilizer where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__QPStabilizer__checkNode" c_CasADi__QPStabilizer__checkNode
  :: Ptr QPStabilizer' -> IO CInt
casADi__QPStabilizer__checkNode
  :: QPStabilizer -> IO Bool
casADi__QPStabilizer__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPStabilizer__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
qpStabilizer_checkNode :: QPStabilizerClass a => a -> IO Bool
qpStabilizer_checkNode x = casADi__QPStabilizer__checkNode (castQPStabilizer x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QPStabilizer__creator" c_CasADi__QPStabilizer__creator
  :: Ptr QPStructure' -> IO (Ptr StabilizedQPSolver')
casADi__QPStabilizer__creator
  :: QPStructure -> IO StabilizedQPSolver
casADi__QPStabilizer__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPStabilizer__creator x0' >>= wrapReturn

-- classy wrapper
qpStabilizer_creator :: QPStructure -> IO StabilizedQPSolver
qpStabilizer_creator = casADi__QPStabilizer__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__QPStabilizer__getSolver" c_CasADi__QPStabilizer__getSolver
  :: Ptr QPStabilizer' -> IO (Ptr QPSolver')
casADi__QPStabilizer__getSolver
  :: QPStabilizer -> IO QPSolver
casADi__QPStabilizer__getSolver x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPStabilizer__getSolver x0' >>= wrapReturn

-- classy wrapper
{-|
>Access underlying QP solver.
-}
qpStabilizer_getSolver :: QPStabilizerClass a => a -> IO QPSolver
qpStabilizer_getSolver x = casADi__QPStabilizer__getSolver (castQPStabilizer x)


-- direct wrapper
foreign import ccall unsafe "CasADi__QPStabilizer__QPStabilizer" c_CasADi__QPStabilizer__QPStabilizer
  :: IO (Ptr QPStabilizer')
casADi__QPStabilizer__QPStabilizer
  :: IO QPStabilizer
casADi__QPStabilizer__QPStabilizer  =
  c_CasADi__QPStabilizer__QPStabilizer  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::QPStabilizer::QPStabilizer()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::QPStabilizer::QPStabilizer(const QPStructure &st)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>st:  Problem structure
-}
qpStabilizer :: IO QPStabilizer
qpStabilizer = casADi__QPStabilizer__QPStabilizer


-- direct wrapper
foreign import ccall unsafe "CasADi__QPStabilizer__QPStabilizer_TIC" c_CasADi__QPStabilizer__QPStabilizer_TIC
  :: Ptr QPStructure' -> IO (Ptr QPStabilizer')
casADi__QPStabilizer__QPStabilizer'
  :: QPStructure -> IO QPStabilizer
casADi__QPStabilizer__QPStabilizer' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__QPStabilizer__QPStabilizer_TIC x0' >>= wrapReturn

-- classy wrapper
qpStabilizer' :: QPStructure -> IO QPStabilizer
qpStabilizer' = casADi__QPStabilizer__QPStabilizer'

