{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SDPSDQPSolver
       (
         SDPSDQPSolver,
         SDPSDQPSolverClass(..),
         sdpsdqpSolver,
         sdpsdqpSolver',
         sdpsdqpSolver_checkNode,
         sdpsdqpSolver_creator,
         sdpsdqpSolver_getSolver,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SDPSDQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSDQPSolver__checkNode" c_CasADi__SDPSDQPSolver__checkNode
  :: Ptr SDPSDQPSolver' -> IO CInt
casADi__SDPSDQPSolver__checkNode
  :: SDPSDQPSolver -> IO Bool
casADi__SDPSDQPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSDQPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
sdpsdqpSolver_checkNode :: SDPSDQPSolverClass a => a -> IO Bool
sdpsdqpSolver_checkNode x = casADi__SDPSDQPSolver__checkNode (castSDPSDQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSDQPSolver__creator" c_CasADi__SDPSDQPSolver__creator
  :: Ptr SDQPStructure' -> IO (Ptr SDQPSolver')
casADi__SDPSDQPSolver__creator
  :: SDQPStructure -> IO SDQPSolver
casADi__SDPSDQPSolver__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSDQPSolver__creator x0' >>= wrapReturn

-- classy wrapper
sdpsdqpSolver_creator :: SDQPStructure -> IO SDQPSolver
sdpsdqpSolver_creator = casADi__SDPSDQPSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSDQPSolver__getSolver" c_CasADi__SDPSDQPSolver__getSolver
  :: Ptr SDPSDQPSolver' -> IO (Ptr SDPSolver')
casADi__SDPSDQPSolver__getSolver
  :: SDPSDQPSolver -> IO SDPSolver
casADi__SDPSDQPSolver__getSolver x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSDQPSolver__getSolver x0' >>= wrapReturn

-- classy wrapper
{-|
>Access underlying SDP solver.
-}
sdpsdqpSolver_getSolver :: SDPSDQPSolverClass a => a -> IO SDPSolver
sdpsdqpSolver_getSolver x = casADi__SDPSDQPSolver__getSolver (castSDPSDQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSDQPSolver__SDPSDQPSolver" c_CasADi__SDPSDQPSolver__SDPSDQPSolver
  :: IO (Ptr SDPSDQPSolver')
casADi__SDPSDQPSolver__SDPSDQPSolver
  :: IO SDPSDQPSolver
casADi__SDPSDQPSolver__SDPSDQPSolver  =
  c_CasADi__SDPSDQPSolver__SDPSDQPSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::SDPSDQPSolver::SDPSDQPSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::SDPSDQPSolver::SDPSDQPSolver(const SDQPStructure &st)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>st:  Problem structure
-}
sdpsdqpSolver :: IO SDPSDQPSolver
sdpsdqpSolver = casADi__SDPSDQPSolver__SDPSDQPSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSDQPSolver__SDPSDQPSolver_TIC" c_CasADi__SDPSDQPSolver__SDPSDQPSolver_TIC
  :: Ptr SDQPStructure' -> IO (Ptr SDPSDQPSolver')
casADi__SDPSDQPSolver__SDPSDQPSolver'
  :: SDQPStructure -> IO SDPSDQPSolver
casADi__SDPSDQPSolver__SDPSDQPSolver' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSDQPSolver__SDPSDQPSolver_TIC x0' >>= wrapReturn

-- classy wrapper
sdpsdqpSolver' :: SDQPStructure -> IO SDPSDQPSolver
sdpsdqpSolver' = casADi__SDPSDQPSolver__SDPSDQPSolver'

