{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SDPSOCPSolver
       (
         SDPSOCPSolver,
         SDPSOCPSolverClass(..),
         sdpsocpSolver,
         sdpsocpSolver',
         sdpsocpSolver_checkNode,
         sdpsocpSolver_creator,
         sdpsocpSolver_getSolver,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SDPSOCPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSOCPSolver__checkNode" c_CasADi__SDPSOCPSolver__checkNode
  :: Ptr SDPSOCPSolver' -> IO CInt
casADi__SDPSOCPSolver__checkNode
  :: SDPSOCPSolver -> IO Bool
casADi__SDPSOCPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSOCPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
sdpsocpSolver_checkNode :: SDPSOCPSolverClass a => a -> IO Bool
sdpsocpSolver_checkNode x = casADi__SDPSOCPSolver__checkNode (castSDPSOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSOCPSolver__creator" c_CasADi__SDPSOCPSolver__creator
  :: Ptr SOCPStructure' -> IO (Ptr SOCPSolver')
casADi__SDPSOCPSolver__creator
  :: SOCPStructure -> IO SOCPSolver
casADi__SDPSOCPSolver__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSOCPSolver__creator x0' >>= wrapReturn

-- classy wrapper
sdpsocpSolver_creator :: SOCPStructure -> IO SOCPSolver
sdpsocpSolver_creator = casADi__SDPSOCPSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSOCPSolver__getSolver" c_CasADi__SDPSOCPSolver__getSolver
  :: Ptr SDPSOCPSolver' -> IO (Ptr SDPSolver')
casADi__SDPSOCPSolver__getSolver
  :: SDPSOCPSolver -> IO SDPSolver
casADi__SDPSOCPSolver__getSolver x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSOCPSolver__getSolver x0' >>= wrapReturn

-- classy wrapper
{-|
>Access underlying SDP solver.
-}
sdpsocpSolver_getSolver :: SDPSOCPSolverClass a => a -> IO SDPSolver
sdpsocpSolver_getSolver x = casADi__SDPSOCPSolver__getSolver (castSDPSOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSOCPSolver__SDPSOCPSolver" c_CasADi__SDPSOCPSolver__SDPSOCPSolver
  :: IO (Ptr SDPSOCPSolver')
casADi__SDPSOCPSolver__SDPSOCPSolver
  :: IO SDPSOCPSolver
casADi__SDPSOCPSolver__SDPSOCPSolver  =
  c_CasADi__SDPSOCPSolver__SDPSOCPSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::SDPSOCPSolver::SDPSOCPSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::SDPSOCPSolver::SDPSOCPSolver(const SOCPStructure &st)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>st:  Problem structure
-}
sdpsocpSolver :: IO SDPSOCPSolver
sdpsocpSolver = casADi__SDPSOCPSolver__SDPSOCPSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSOCPSolver__SDPSOCPSolver_TIC" c_CasADi__SDPSOCPSolver__SDPSOCPSolver_TIC
  :: Ptr SOCPStructure' -> IO (Ptr SDPSOCPSolver')
casADi__SDPSOCPSolver__SDPSOCPSolver'
  :: SOCPStructure -> IO SDPSOCPSolver
casADi__SDPSOCPSolver__SDPSOCPSolver' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSOCPSolver__SDPSOCPSolver_TIC x0' >>= wrapReturn

-- classy wrapper
sdpsocpSolver' :: SOCPStructure -> IO SDPSOCPSolver
sdpsocpSolver' = casADi__SDPSOCPSolver__SDPSOCPSolver'

