{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SDPSolver
       (
         SDPSolver,
         SDPSolverClass(..),
         sdpSolver,
         sdpSolver_checkNode,
         sdpSolver_setSOCPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SDPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSolver__checkNode" c_CasADi__SDPSolver__checkNode
  :: Ptr SDPSolver' -> IO CInt
casADi__SDPSolver__checkNode
  :: SDPSolver -> IO Bool
casADi__SDPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
sdpSolver_checkNode :: SDPSolverClass a => a -> IO Bool
sdpSolver_checkNode x = casADi__SDPSolver__checkNode (castSDPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSolver__setSOCPOptions" c_CasADi__SDPSolver__setSOCPOptions
  :: Ptr SDPSolver' -> IO ()
casADi__SDPSolver__setSOCPOptions
  :: SDPSolver -> IO ()
casADi__SDPSolver__setSOCPOptions x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDPSolver__setSOCPOptions x0' >>= wrapReturn

-- classy wrapper
{-|
>Set options that make the SDP solver more suitable for solving SOCPs.
-}
sdpSolver_setSOCPOptions :: SDPSolverClass a => a -> IO ()
sdpSolver_setSOCPOptions x = casADi__SDPSolver__setSOCPOptions (castSDPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDPSolver__SDPSolver" c_CasADi__SDPSolver__SDPSolver
  :: IO (Ptr SDPSolver')
casADi__SDPSolver__SDPSolver
  :: IO SDPSolver
casADi__SDPSolver__SDPSolver  =
  c_CasADi__SDPSolver__SDPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
sdpSolver :: IO SDPSolver
sdpSolver = casADi__SDPSolver__SDPSolver

