{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SDQPSolver
       (
         SDQPSolver,
         SDQPSolverClass(..),
         sdqpSolver,
         sdqpSolver_checkNode,
         sdqpSolver_setSOCQPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SDQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SDQPSolver__checkNode" c_CasADi__SDQPSolver__checkNode
  :: Ptr SDQPSolver' -> IO CInt
casADi__SDQPSolver__checkNode
  :: SDQPSolver -> IO Bool
casADi__SDQPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDQPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
sdqpSolver_checkNode :: SDQPSolverClass a => a -> IO Bool
sdqpSolver_checkNode x = casADi__SDQPSolver__checkNode (castSDQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDQPSolver__setSOCQPOptions" c_CasADi__SDQPSolver__setSOCQPOptions
  :: Ptr SDQPSolver' -> IO ()
casADi__SDQPSolver__setSOCQPOptions
  :: SDQPSolver -> IO ()
casADi__SDQPSolver__setSOCQPOptions x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SDQPSolver__setSOCQPOptions x0' >>= wrapReturn

-- classy wrapper
{-|
>Set options that make the SDQP solver more suitable for solving SOCPs.
-}
sdqpSolver_setSOCQPOptions :: SDQPSolverClass a => a -> IO ()
sdqpSolver_setSOCQPOptions x = casADi__SDQPSolver__setSOCQPOptions (castSDQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SDQPSolver__SDQPSolver" c_CasADi__SDQPSolver__SDQPSolver
  :: IO (Ptr SDQPSolver')
casADi__SDQPSolver__SDQPSolver
  :: IO SDQPSolver
casADi__SDQPSolver__SDQPSolver  =
  c_CasADi__SDQPSolver__SDQPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
sdqpSolver :: IO SDQPSolver
sdqpSolver = casADi__SDQPSolver__SDQPSolver

