{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SOCPQCQPSolver
       (
         SOCPQCQPSolver,
         SOCPQCQPSolverClass(..),
         socpqcqpSolver,
         socpqcqpSolver',
         socpqcqpSolver_checkNode,
         socpqcqpSolver_creator,
         socpqcqpSolver_getSolver,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SOCPQCQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SOCPQCQPSolver__checkNode" c_CasADi__SOCPQCQPSolver__checkNode
  :: Ptr SOCPQCQPSolver' -> IO CInt
casADi__SOCPQCQPSolver__checkNode
  :: SOCPQCQPSolver -> IO Bool
casADi__SOCPQCQPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SOCPQCQPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
socpqcqpSolver_checkNode :: SOCPQCQPSolverClass a => a -> IO Bool
socpqcqpSolver_checkNode x = casADi__SOCPQCQPSolver__checkNode (castSOCPQCQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SOCPQCQPSolver__creator" c_CasADi__SOCPQCQPSolver__creator
  :: Ptr QCQPStructure' -> IO (Ptr QCQPSolver')
casADi__SOCPQCQPSolver__creator
  :: QCQPStructure -> IO QCQPSolver
casADi__SOCPQCQPSolver__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SOCPQCQPSolver__creator x0' >>= wrapReturn

-- classy wrapper
socpqcqpSolver_creator :: QCQPStructure -> IO QCQPSolver
socpqcqpSolver_creator = casADi__SOCPQCQPSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__SOCPQCQPSolver__getSolver" c_CasADi__SOCPQCQPSolver__getSolver
  :: Ptr SOCPQCQPSolver' -> IO (Ptr SOCPSolver')
casADi__SOCPQCQPSolver__getSolver
  :: SOCPQCQPSolver -> IO SOCPSolver
casADi__SOCPQCQPSolver__getSolver x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SOCPQCQPSolver__getSolver x0' >>= wrapReturn

-- classy wrapper
{-|
>Access underlying SOCP solver.
-}
socpqcqpSolver_getSolver :: SOCPQCQPSolverClass a => a -> IO SOCPSolver
socpqcqpSolver_getSolver x = casADi__SOCPQCQPSolver__getSolver (castSOCPQCQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SOCPQCQPSolver__SOCPQCQPSolver" c_CasADi__SOCPQCQPSolver__SOCPQCQPSolver
  :: IO (Ptr SOCPQCQPSolver')
casADi__SOCPQCQPSolver__SOCPQCQPSolver
  :: IO SOCPQCQPSolver
casADi__SOCPQCQPSolver__SOCPQCQPSolver  =
  c_CasADi__SOCPQCQPSolver__SOCPQCQPSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::SOCPQCQPSolver::SOCPQCQPSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::SOCPQCQPSolver::SOCPQCQPSolver(const QCQPStructure &st)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>st:  Problem structure
-}
socpqcqpSolver :: IO SOCPQCQPSolver
socpqcqpSolver = casADi__SOCPQCQPSolver__SOCPQCQPSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__SOCPQCQPSolver__SOCPQCQPSolver_TIC" c_CasADi__SOCPQCQPSolver__SOCPQCQPSolver_TIC
  :: Ptr QCQPStructure' -> IO (Ptr SOCPQCQPSolver')
casADi__SOCPQCQPSolver__SOCPQCQPSolver'
  :: QCQPStructure -> IO SOCPQCQPSolver
casADi__SOCPQCQPSolver__SOCPQCQPSolver' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SOCPQCQPSolver__SOCPQCQPSolver_TIC x0' >>= wrapReturn

-- classy wrapper
socpqcqpSolver' :: QCQPStructure -> IO SOCPQCQPSolver
socpqcqpSolver' = casADi__SOCPQCQPSolver__SOCPQCQPSolver'

