{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SOCPSolver
       (
         SOCPSolver,
         SOCPSolverClass(..),
         socpSolver,
         socpSolver_checkNode,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SOCPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SOCPSolver__checkNode" c_CasADi__SOCPSolver__checkNode
  :: Ptr SOCPSolver' -> IO CInt
casADi__SOCPSolver__checkNode
  :: SOCPSolver -> IO Bool
casADi__SOCPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SOCPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
socpSolver_checkNode :: SOCPSolverClass a => a -> IO Bool
socpSolver_checkNode x = casADi__SOCPSolver__checkNode (castSOCPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SOCPSolver__SOCPSolver" c_CasADi__SOCPSolver__SOCPSolver
  :: IO (Ptr SOCPSolver')
casADi__SOCPSolver__SOCPSolver
  :: IO SOCPSolver
casADi__SOCPSolver__SOCPSolver  =
  c_CasADi__SOCPSolver__SOCPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
socpSolver :: IO SOCPSolver
socpSolver = casADi__SOCPSolver__SOCPSolver

