{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SQPMethod
       (
         SQPMethod,
         SQPMethodClass(..),
         sqpMethod,
         sqpMethod',
         sqpMethod_checkNode,
         sqpMethod_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SQPMethod where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SQPMethod__checkNode" c_CasADi__SQPMethod__checkNode
  :: Ptr SQPMethod' -> IO CInt
casADi__SQPMethod__checkNode
  :: SQPMethod -> IO Bool
casADi__SQPMethod__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SQPMethod__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
sqpMethod_checkNode :: SQPMethodClass a => a -> IO Bool
sqpMethod_checkNode x = casADi__SQPMethod__checkNode (castSQPMethod x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SQPMethod__creator" c_CasADi__SQPMethod__creator
  :: Ptr Function' -> IO (Ptr NLPSolver')
casADi__SQPMethod__creator
  :: Function -> IO NLPSolver
casADi__SQPMethod__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SQPMethod__creator x0' >>= wrapReturn

-- classy wrapper
sqpMethod_creator :: Function -> IO NLPSolver
sqpMethod_creator = casADi__SQPMethod__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__SQPMethod__SQPMethod" c_CasADi__SQPMethod__SQPMethod
  :: IO (Ptr SQPMethod')
casADi__SQPMethod__SQPMethod
  :: IO SQPMethod
casADi__SQPMethod__SQPMethod  =
  c_CasADi__SQPMethod__SQPMethod  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::SQPMethod::SQPMethod()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::SQPMethod::SQPMethod(const Function &F, const Function &G)
>------------------------------------------------------------------------
>
>[DEPRECATED] Create an NLP solver instance (legacy syntax)
>
>>  CasADi::SQPMethod::SQPMethod(const Function &nlp)
>------------------------------------------------------------------------
>
>Create an NLP solver instance.
-}
sqpMethod :: IO SQPMethod
sqpMethod = casADi__SQPMethod__SQPMethod


-- direct wrapper
foreign import ccall unsafe "CasADi__SQPMethod__SQPMethod_TIC" c_CasADi__SQPMethod__SQPMethod_TIC
  :: Ptr Function' -> IO (Ptr SQPMethod')
casADi__SQPMethod__SQPMethod'
  :: Function -> IO SQPMethod
casADi__SQPMethod__SQPMethod' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SQPMethod__SQPMethod_TIC x0' >>= wrapReturn

-- classy wrapper
sqpMethod' :: Function -> IO SQPMethod
sqpMethod' = casADi__SQPMethod__SQPMethod'

