{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SimpleHomotopyNLPSolver
       (
         SimpleHomotopyNLPSolver,
         SimpleHomotopyNLPSolverClass(..),
         simpleHomotopyNLPSolver,
         simpleHomotopyNLPSolver',
         simpleHomotopyNLPSolver_checkNode,
         simpleHomotopyNLPSolver_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SimpleHomotopyNLPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleHomotopyNLPSolver__checkNode" c_CasADi__SimpleHomotopyNLPSolver__checkNode
  :: Ptr SimpleHomotopyNLPSolver' -> IO CInt
casADi__SimpleHomotopyNLPSolver__checkNode
  :: SimpleHomotopyNLPSolver -> IO Bool
casADi__SimpleHomotopyNLPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SimpleHomotopyNLPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
simpleHomotopyNLPSolver_checkNode :: SimpleHomotopyNLPSolverClass a => a -> IO Bool
simpleHomotopyNLPSolver_checkNode x = casADi__SimpleHomotopyNLPSolver__checkNode (castSimpleHomotopyNLPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleHomotopyNLPSolver__creator" c_CasADi__SimpleHomotopyNLPSolver__creator
  :: Ptr Function' -> IO (Ptr HomotopyNLPSolver')
casADi__SimpleHomotopyNLPSolver__creator
  :: Function -> IO HomotopyNLPSolver
casADi__SimpleHomotopyNLPSolver__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SimpleHomotopyNLPSolver__creator x0' >>= wrapReturn

-- classy wrapper
simpleHomotopyNLPSolver_creator :: Function -> IO HomotopyNLPSolver
simpleHomotopyNLPSolver_creator = casADi__SimpleHomotopyNLPSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver" c_CasADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver
  :: IO (Ptr SimpleHomotopyNLPSolver')
casADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver
  :: IO SimpleHomotopyNLPSolver
casADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver  =
  c_CasADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::SimpleHomotopyNLPSolver::SimpleHomotopyNLPSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::SimpleHomotopyNLPSolver::SimpleHomotopyNLPSolver(const Function &hnlp)
>------------------------------------------------------------------------
>
>Create an NLP solver instance.
-}
simpleHomotopyNLPSolver :: IO SimpleHomotopyNLPSolver
simpleHomotopyNLPSolver = casADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver_TIC" c_CasADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver_TIC
  :: Ptr Function' -> IO (Ptr SimpleHomotopyNLPSolver')
casADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver'
  :: Function -> IO SimpleHomotopyNLPSolver
casADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver_TIC x0' >>= wrapReturn

-- classy wrapper
simpleHomotopyNLPSolver' :: Function -> IO SimpleHomotopyNLPSolver
simpleHomotopyNLPSolver' = casADi__SimpleHomotopyNLPSolver__SimpleHomotopyNLPSolver'

