{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.SimpleIndefDpleSolver
       (
         SimpleIndefDpleSolver,
         SimpleIndefDpleSolverClass(..),
         simpleIndefDpleSolver,
         simpleIndefDpleSolver',
         simpleIndefDpleSolver_checkNode,
         simpleIndefDpleSolver_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show SimpleIndefDpleSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleIndefDpleSolver__checkNode" c_CasADi__SimpleIndefDpleSolver__checkNode
  :: Ptr SimpleIndefDpleSolver' -> IO CInt
casADi__SimpleIndefDpleSolver__checkNode
  :: SimpleIndefDpleSolver -> IO Bool
casADi__SimpleIndefDpleSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__SimpleIndefDpleSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
simpleIndefDpleSolver_checkNode :: SimpleIndefDpleSolverClass a => a -> IO Bool
simpleIndefDpleSolver_checkNode x = casADi__SimpleIndefDpleSolver__checkNode (castSimpleIndefDpleSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleIndefDpleSolver__creator" c_CasADi__SimpleIndefDpleSolver__creator
  :: Ptr (CppVec (Ptr Sparsity')) -> Ptr (CppVec (Ptr Sparsity')) -> IO (Ptr DpleSolver')
casADi__SimpleIndefDpleSolver__creator
  :: Vector Sparsity -> Vector Sparsity -> IO DpleSolver
casADi__SimpleIndefDpleSolver__creator x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__SimpleIndefDpleSolver__creator x0' x1' >>= wrapReturn

-- classy wrapper
simpleIndefDpleSolver_creator :: Vector Sparsity -> Vector Sparsity -> IO DpleSolver
simpleIndefDpleSolver_creator = casADi__SimpleIndefDpleSolver__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver" c_CasADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver
  :: IO (Ptr SimpleIndefDpleSolver')
casADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver
  :: IO SimpleIndefDpleSolver
casADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver  =
  c_CasADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::SimpleIndefDpleSolver::SimpleIndefDpleSolver()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::SimpleIndefDpleSolver::SimpleIndefDpleSolver(const std::vector< Sparsity > &A, const std::vector< Sparsity > &V)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>A:  List of sparsities of A_i
>
>V:  List of sparsities of V_i
-}
simpleIndefDpleSolver :: IO SimpleIndefDpleSolver
simpleIndefDpleSolver = casADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver


-- direct wrapper
foreign import ccall unsafe "CasADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver_TIC" c_CasADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver_TIC
  :: Ptr (CppVec (Ptr Sparsity')) -> Ptr (CppVec (Ptr Sparsity')) -> IO (Ptr SimpleIndefDpleSolver')
casADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver'
  :: Vector Sparsity -> Vector Sparsity -> IO SimpleIndefDpleSolver
casADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver_TIC x0' x1' >>= wrapReturn

-- classy wrapper
simpleIndefDpleSolver' :: Vector Sparsity -> Vector Sparsity -> IO SimpleIndefDpleSolver
simpleIndefDpleSolver' = casADi__SimpleIndefDpleSolver__SimpleIndefDpleSolver'

