{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.Simulator
       (
         Simulator,
         SimulatorClass(..),
         simulator,
         simulator',
         simulator'',
         simulator''',
         simulator'''',
         simulator_checkNode,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show Simulator where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__Simulator__checkNode" c_CasADi__Simulator__checkNode
  :: Ptr Simulator' -> IO CInt
casADi__Simulator__checkNode
  :: Simulator -> IO Bool
casADi__Simulator__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__Simulator__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
simulator_checkNode :: SimulatorClass a => a -> IO Bool
simulator_checkNode x = casADi__Simulator__checkNode (castSimulator x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Simulator__Simulator" c_CasADi__Simulator__Simulator
  :: IO (Ptr Simulator')
casADi__Simulator__Simulator
  :: IO Simulator
casADi__Simulator__Simulator  =
  c_CasADi__Simulator__Simulator  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::Simulator::Simulator()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::Simulator::Simulator(const Integrator &integrator, const Function &output_fcn, const std::vector< double > &grid)
>------------------------------------------------------------------------
>
>Constructor.
>
>Parameters:
>-----------
>
>output_fcn:  output function which maps to n outputs.
>
>>Input scheme: CasADi::DAEInput (DAE_NUM_IN = 5) [daeIn]
>+-----------+-------+----------------------------+
>| Full name | Short |        Description         |
>+===========+=======+============================+
>| DAE_X     | x     | Differential state .       |
>+-----------+-------+----------------------------+
>| DAE_Z     | z     | Algebraic state .          |
>+-----------+-------+----------------------------+
>| DAE_P     | p     | Parameter .                |
>+-----------+-------+----------------------------+
>| DAE_T     | t     | Explicit time dependence . |
>+-----------+-------+----------------------------+
>
>>  CasADi::Simulator::Simulator(const Integrator &integrator, const std::vector< double > &grid)
>------------------------------------------------------------------------
>
>Output function equal to the state.
-}
simulator :: IO Simulator
simulator = casADi__Simulator__Simulator


-- direct wrapper
foreign import ccall unsafe "CasADi__Simulator__Simulator_TIC" c_CasADi__Simulator__Simulator_TIC
  :: Ptr Integrator' -> Ptr Function' -> Ptr (CppVec CDouble) -> IO (Ptr Simulator')
casADi__Simulator__Simulator'
  :: Integrator -> Function -> Vector Double -> IO Simulator
casADi__Simulator__Simulator' x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__Simulator__Simulator_TIC x0' x1' x2' >>= wrapReturn

-- classy wrapper
simulator' :: Integrator -> Function -> Vector Double -> IO Simulator
simulator' = casADi__Simulator__Simulator'


-- direct wrapper
foreign import ccall unsafe "CasADi__Simulator__Simulator_TIC_TIC" c_CasADi__Simulator__Simulator_TIC_TIC
  :: Ptr Integrator' -> Ptr Function' -> Ptr DMatrix' -> IO (Ptr Simulator')
casADi__Simulator__Simulator''
  :: Integrator -> Function -> DMatrix -> IO Simulator
casADi__Simulator__Simulator'' x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__Simulator__Simulator_TIC_TIC x0' x1' x2' >>= wrapReturn

-- classy wrapper
simulator'' :: Integrator -> Function -> DMatrix -> IO Simulator
simulator'' = casADi__Simulator__Simulator''


-- direct wrapper
foreign import ccall unsafe "CasADi__Simulator__Simulator_TIC_TIC_TIC" c_CasADi__Simulator__Simulator_TIC_TIC_TIC
  :: Ptr Integrator' -> Ptr (CppVec CDouble) -> IO (Ptr Simulator')
casADi__Simulator__Simulator'''
  :: Integrator -> Vector Double -> IO Simulator
casADi__Simulator__Simulator''' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__Simulator__Simulator_TIC_TIC_TIC x0' x1' >>= wrapReturn

-- classy wrapper
simulator''' :: Integrator -> Vector Double -> IO Simulator
simulator''' = casADi__Simulator__Simulator'''


-- direct wrapper
foreign import ccall unsafe "CasADi__Simulator__Simulator_TIC_TIC_TIC_TIC" c_CasADi__Simulator__Simulator_TIC_TIC_TIC_TIC
  :: Ptr Integrator' -> Ptr DMatrix' -> IO (Ptr Simulator')
casADi__Simulator__Simulator''''
  :: Integrator -> DMatrix -> IO Simulator
casADi__Simulator__Simulator'''' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__Simulator__Simulator_TIC_TIC_TIC_TIC x0' x1' >>= wrapReturn

-- classy wrapper
simulator'''' :: Integrator -> DMatrix -> IO Simulator
simulator'''' = casADi__Simulator__Simulator''''

