{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.StabilizedQPSolver
       (
         StabilizedQPSolver,
         StabilizedQPSolverClass(..),
         stabilizedQPSolver,
         stabilizedQPSolver_checkNode,
         stabilizedQPSolver_generateNativeCode,
         stabilizedQPSolver_setLPOptions,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show StabilizedQPSolver where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedQPSolver__checkNode" c_CasADi__StabilizedQPSolver__checkNode
  :: Ptr StabilizedQPSolver' -> IO CInt
casADi__StabilizedQPSolver__checkNode
  :: StabilizedQPSolver -> IO Bool
casADi__StabilizedQPSolver__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__StabilizedQPSolver__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
stabilizedQPSolver_checkNode :: StabilizedQPSolverClass a => a -> IO Bool
stabilizedQPSolver_checkNode x = casADi__StabilizedQPSolver__checkNode (castStabilizedQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedQPSolver__setLPOptions" c_CasADi__StabilizedQPSolver__setLPOptions
  :: Ptr StabilizedQPSolver' -> IO ()
casADi__StabilizedQPSolver__setLPOptions
  :: StabilizedQPSolver -> IO ()
casADi__StabilizedQPSolver__setLPOptions x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__StabilizedQPSolver__setLPOptions x0' >>= wrapReturn

-- classy wrapper
{-|
>Set options that make the QP solver more suitable for solving LPs.
-}
stabilizedQPSolver_setLPOptions :: StabilizedQPSolverClass a => a -> IO ()
stabilizedQPSolver_setLPOptions x = casADi__StabilizedQPSolver__setLPOptions (castStabilizedQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedQPSolver__generateNativeCode" c_CasADi__StabilizedQPSolver__generateNativeCode
  :: Ptr StabilizedQPSolver' -> Ptr StdString' -> IO ()
casADi__StabilizedQPSolver__generateNativeCode
  :: StabilizedQPSolver -> String -> IO ()
casADi__StabilizedQPSolver__generateNativeCode x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__StabilizedQPSolver__generateNativeCode x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Generate native code in the interfaced language for debugging
-}
stabilizedQPSolver_generateNativeCode :: StabilizedQPSolverClass a => a -> String -> IO ()
stabilizedQPSolver_generateNativeCode x = casADi__StabilizedQPSolver__generateNativeCode (castStabilizedQPSolver x)


-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedQPSolver__StabilizedQPSolver" c_CasADi__StabilizedQPSolver__StabilizedQPSolver
  :: IO (Ptr StabilizedQPSolver')
casADi__StabilizedQPSolver__StabilizedQPSolver
  :: IO StabilizedQPSolver
casADi__StabilizedQPSolver__StabilizedQPSolver  =
  c_CasADi__StabilizedQPSolver__StabilizedQPSolver  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
stabilizedQPSolver :: IO StabilizedQPSolver
stabilizedQPSolver = casADi__StabilizedQPSolver__StabilizedQPSolver

