{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.StabilizedSQPMethod
       (
         StabilizedSQPMethod,
         StabilizedSQPMethodClass(..),
         stabilizedSQPMethod,
         stabilizedSQPMethod',
         stabilizedSQPMethod_checkNode,
         stabilizedSQPMethod_creator,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show StabilizedSQPMethod where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedSQPMethod__checkNode" c_CasADi__StabilizedSQPMethod__checkNode
  :: Ptr StabilizedSQPMethod' -> IO CInt
casADi__StabilizedSQPMethod__checkNode
  :: StabilizedSQPMethod -> IO Bool
casADi__StabilizedSQPMethod__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__StabilizedSQPMethod__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Check if the node is pointing to the right type of object.
-}
stabilizedSQPMethod_checkNode :: StabilizedSQPMethodClass a => a -> IO Bool
stabilizedSQPMethod_checkNode x = casADi__StabilizedSQPMethod__checkNode (castStabilizedSQPMethod x)


-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedSQPMethod__creator" c_CasADi__StabilizedSQPMethod__creator
  :: Ptr Function' -> IO (Ptr NLPSolver')
casADi__StabilizedSQPMethod__creator
  :: Function -> IO NLPSolver
casADi__StabilizedSQPMethod__creator x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__StabilizedSQPMethod__creator x0' >>= wrapReturn

-- classy wrapper
stabilizedSQPMethod_creator :: Function -> IO NLPSolver
stabilizedSQPMethod_creator = casADi__StabilizedSQPMethod__creator


-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedSQPMethod__StabilizedSQPMethod" c_CasADi__StabilizedSQPMethod__StabilizedSQPMethod
  :: IO (Ptr StabilizedSQPMethod')
casADi__StabilizedSQPMethod__StabilizedSQPMethod
  :: IO StabilizedSQPMethod
casADi__StabilizedSQPMethod__StabilizedSQPMethod  =
  c_CasADi__StabilizedSQPMethod__StabilizedSQPMethod  >>= wrapReturn

-- classy wrapper
{-|
>>  CasADi::StabilizedSQPMethod::StabilizedSQPMethod()
>------------------------------------------------------------------------
>
>Default constructor.
>
>>  CasADi::StabilizedSQPMethod::StabilizedSQPMethod(const Function &F, const Function &G)
>------------------------------------------------------------------------
>
>[DEPRECATED] Create an NLP solver instance (legacy syntax)
>
>>  CasADi::StabilizedSQPMethod::StabilizedSQPMethod(const Function &nlp)
>------------------------------------------------------------------------
>
>Create an NLP solver instance.
-}
stabilizedSQPMethod :: IO StabilizedSQPMethod
stabilizedSQPMethod = casADi__StabilizedSQPMethod__StabilizedSQPMethod


-- direct wrapper
foreign import ccall unsafe "CasADi__StabilizedSQPMethod__StabilizedSQPMethod_TIC" c_CasADi__StabilizedSQPMethod__StabilizedSQPMethod_TIC
  :: Ptr Function' -> IO (Ptr StabilizedSQPMethod')
casADi__StabilizedSQPMethod__StabilizedSQPMethod'
  :: Function -> IO StabilizedSQPMethod
casADi__StabilizedSQPMethod__StabilizedSQPMethod' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__StabilizedSQPMethod__StabilizedSQPMethod_TIC x0' >>= wrapReturn

-- classy wrapper
stabilizedSQPMethod' :: Function -> IO StabilizedSQPMethod
stabilizedSQPMethod' = casADi__StabilizedSQPMethod__StabilizedSQPMethod'

