{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.Variable
       (
         Variable,
         VariableClass(..),
         variable,
         variable_atTime,
         variable_atTime',
         variable_atTime'',
         variable_atTime''',
         variable_name,
         variable_setName,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show Variable where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__Variable__name" c_CasADi__Variable__name
  :: Ptr Variable' -> IO (Ptr StdString')
casADi__Variable__name
  :: Variable -> IO String
casADi__Variable__name x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__Variable__name x0' >>= wrapReturn

-- classy wrapper
{-|
>Variable name.
-}
variable_name :: VariableClass a => a -> IO String
variable_name x = casADi__Variable__name (castVariable x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Variable__setName" c_CasADi__Variable__setName
  :: Ptr Variable' -> Ptr StdString' -> IO ()
casADi__Variable__setName
  :: Variable -> String -> IO ()
casADi__Variable__setName x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__Variable__setName x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Set the variable name (and corresponding expressions)
-}
variable_setName :: VariableClass a => a -> String -> IO ()
variable_setName x = casADi__Variable__setName (castVariable x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Variable__atTime" c_CasADi__Variable__atTime
  :: Ptr Variable' -> CDouble -> CInt -> IO (Ptr SXElement')
casADi__Variable__atTime
  :: Variable -> Double -> Bool -> IO SXElement
casADi__Variable__atTime x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__Variable__atTime x0' x1' x2' >>= wrapReturn

-- classy wrapper
{-|
>>  SXElement CasADi::Variable::atTime(double t, bool allocate=false) const 
>------------------------------------------------------------------------
>
>Timed variable (never allocate)
>
>>  SXElement CasADi::Variable::atTime(double t, bool allocate=false)
>------------------------------------------------------------------------
>
>Timed variable (allocate if necessary)
-}
variable_atTime :: VariableClass a => a -> Double -> Bool -> IO SXElement
variable_atTime x = casADi__Variable__atTime (castVariable x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Variable__atTime_TIC" c_CasADi__Variable__atTime_TIC
  :: Ptr Variable' -> CDouble -> IO (Ptr SXElement')
casADi__Variable__atTime'
  :: Variable -> Double -> IO SXElement
casADi__Variable__atTime' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__Variable__atTime_TIC x0' x1' >>= wrapReturn

-- classy wrapper
variable_atTime' :: VariableClass a => a -> Double -> IO SXElement
variable_atTime' x = casADi__Variable__atTime' (castVariable x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Variable__atTime_TIC_TIC" c_CasADi__Variable__atTime_TIC_TIC
  :: Ptr Variable' -> CDouble -> CInt -> IO (Ptr SXElement')
casADi__Variable__atTime''
  :: Variable -> Double -> Bool -> IO SXElement
casADi__Variable__atTime'' x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__Variable__atTime_TIC_TIC x0' x1' x2' >>= wrapReturn

-- classy wrapper
variable_atTime'' :: VariableClass a => a -> Double -> Bool -> IO SXElement
variable_atTime'' x = casADi__Variable__atTime'' (castVariable x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Variable__atTime_TIC_TIC_TIC" c_CasADi__Variable__atTime_TIC_TIC_TIC
  :: Ptr Variable' -> CDouble -> IO (Ptr SXElement')
casADi__Variable__atTime'''
  :: Variable -> Double -> IO SXElement
casADi__Variable__atTime''' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__Variable__atTime_TIC_TIC_TIC x0' x1' >>= wrapReturn

-- classy wrapper
variable_atTime''' :: VariableClass a => a -> Double -> IO SXElement
variable_atTime''' x = casADi__Variable__atTime''' (castVariable x)


-- direct wrapper
foreign import ccall unsafe "CasADi__Variable__Variable" c_CasADi__Variable__Variable
  :: IO (Ptr Variable')
casADi__Variable__Variable
  :: IO Variable
casADi__Variable__Variable  =
  c_CasADi__Variable__Variable  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
variable :: IO Variable
variable = casADi__Variable__Variable

